package hero.client.grapheditor;

/**
 * WFManager.java
 *
 *
 * Created: Fri Aug 09 16:35:03 2002
 *
 * @version 1.0
 */

import hero.net.ProjectSession.StrutsNodeValue;

import hero.util.EventConstants;
import hero.util.JMSServices;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.jms.Message;
import javax.jms.MessageListener;
import javax.swing.JFrame;
import javax.swing.JOptionPane;


public class WFManager {

    Frame frame;
    WFGraph wfGraph;
    WFWindowManager windowManager;
    WFPersistence wfPersistence;
    boolean automatic=true;
    Vector clipboardEdges;
    int clipboardNodes;
    String activeProject=null;    
    String activeUser;
    Listener ml;

    class Edge{
	public int source;
	public int target;
	public Edge(){}
    }


    public class Listener implements MessageListener 
    { 
	private JMSServices jms;
	
	public Listener(){} 
	
	public void subscription(String subs) throws Exception 
	{
	    jms = JMSServices.getInstance(); 
	    jms.createSubscription(this,subs); 
	}

	public void onMessage(Message message) 
	{ 
	    Object key; 
	    Object value = Color.lightGray; 

	    synchronized(this){

	    try{ 
		Hashtable evt = 	JMSServices.getEvt(message); 

		if (((String)evt.get(EventConstants.PROJECTNAME)).equals(activeProject))
		{
		    String event = (String)evt.get(EventConstants.EVENT); 
		
		if (event.equals(EventConstants.SETNODESTATE))  
		    { 
			String nodeState = null; 
			String node = (String)evt.get(EventConstants.NODENAME); 
			int state = (new Integer ((String)evt.get(EventConstants.NODESTATE))).intValue();
			wfGraph.changeNodeState(node,state);			
			wfPersistence.setProjectNodeState(node,state);
		    } 
		if (event.equals(EventConstants.ADDNODE)) 
		    {
 			String node = (String)evt.get(EventConstants.NODENAME); 
			String user = (String)evt.get(EventConstants.USERNAME); 
 			String state = (String)evt.get(EventConstants.NODESTATE); 
			wfGraph.addNewNode(node, (new Integer(state)).intValue());			
		    } 
		if (event.equals(EventConstants.ADDEDGE)) 
		    { 
			String edge = (String)evt.get(EventConstants.EDGENAME); 
			String user = (String)evt.get(EventConstants.USERNAME); 

			String nodeIn = (String)evt.get(EventConstants.NODEIN); 
			String nodeOut = (String)evt.get(EventConstants.NODEOUT); 
			wfGraph.restoreEdge(edge,nodeIn,nodeOut);

		    } 
		if (event.equals(EventConstants.DELETENODE)) 
		    { 
			String node = (String)evt.get(EventConstants.NODENAME); 
			String user = (String)evt.get(EventConstants.USERNAME);
			wfGraph.deleteNode(node);
		    } 
		if (event.equals(EventConstants.DELETEEDGE)) 
		    { 
			String edge = (String)evt.get(EventConstants.EDGENAME); 
			String user = (String)evt.get(EventConstants.USERNAME); 
			wfGraph.deleteEdge(edge);
		    } 
		}
		
	    }catch(Exception e1){e1.printStackTrace();} 

	    }

	} 
    }
    
    public void makeJMSConnection(String name)
    {
	try {
	    ml.subscription("projectName = '"+name+"'");
	}catch (Exception e) {e.printStackTrace();}
    }

	public WFManager(Frame frame, WFPersistence persist) {
		try {
			this.frame = frame;
			wfPersistence = persist;
			windowManager = new WFWindowManager(this, wfPersistence);
			clipboardNodes = 0;
			clipboardEdges = new Vector();
			activeUser = wfPersistence.getUser();
			ml = new Listener();
		} catch (Exception e1) {
			e1.printStackTrace();
		}
	}

    public void setWFGraph(WFGraph wfGraph){
	this.wfGraph=wfGraph;
    }

    public void addEdge(){
	wfGraph.setAddEdgeState(true);
    }
    
	public void setEdgeCondition(String edgeName) {
		try {
			String condition = windowManager.setEdgeCondition();
			wfPersistence.setEdgeCondition(edgeName, condition);
		} catch (Exception e1) {
			JOptionPane.showMessageDialog(null, "System Error in set Edge condition");
		}
	}

	public void addProperty(String nodeName) {
		windowManager.addPropertyWindow(nodeName);
	}
	
	public void setNodeHook(String nodeName) {
		try {
			Vector hook = windowManager.addNodeInterHook();
			if (hook.size()!=0)
			{
			  wfPersistence.addNodeInterHook(nodeName, (String) hook.get(0), (String) hook.get(1), (String) hook.get(2));
			  String value = windowManager.setNodeInterHookValue((String) hook.get(2));
			  wfPersistence.setNodeInterHookValue(nodeName, (String) hook.get(0), value);
			}
		} catch (Exception e1) {
			JOptionPane.showMessageDialog(null, "System Error in add hook: the hook already exist");
		}
	}
	
	public void editNode(String nodeName) {
				try {
					 windowManager.editNode(nodeName);						 							
				} catch (Exception e1) {
					JOptionPane.showMessageDialog(null, "System Error in set description");
				}
			}

	public void addNode() {
			try {
				wfGraph.setAddEdgeState(false);
				Vector addNode = windowManager.addNode();
				String name = (String) addNode.elementAt(0);
				if (!name.equalsIgnoreCase("")) {
					wfPersistence.addNode(name, (String) addNode.elementAt(1));
					//	    wfGraph.addNewNode(name);			
				}
				updateStatus();
			} catch (Exception e1) {}
		}

    
    public void clearSelection(){
	wfGraph.clearSelection();
    }
    public void selectAll(){
	wfGraph.selectAll();
    }
    
    public void exit(){
	frame.dispose();
    }

   public void deleteSelection() {
	try {
		Object[] nodes = wfGraph.getSelectionNodes();
		Object[] edges = wfGraph.getSelectionEdges();
		if (edges.length > 0)
			wfPersistence.deleteEdges(edges);
		for (int i = 0; i < nodes.length; i++) {
			wfPersistence.deleteNode((String) nodes[i]);				
		}		
	} catch (Exception e1) {
		JOptionPane.showMessageDialog(null, "You can not delete this selection");
	}

}

    
    public void copySelection(){
	clipboardEdges=new Vector();
	clipboardNodes=((Object[])wfGraph.getSelectionNodes()).length;
       	Object[] nodes=wfGraph.getVertices(wfGraph.getSelectionCells());
	if(nodes.length>1){
	    for(int i=0;i<nodes.length;i++){
		for(int j=0;j<nodes.length;j++){
		    Object[] edge = wfGraph.getEdgesBetween(nodes[i],nodes[j]);		    
		    if(edge.length>0){
			Object target=wfGraph.getSourceVertex(edge[0]);
			if(nodes[i].equals(target)){
			    Edge e= new Edge();
			    e.target=i;
			    e.source=j;
 			    clipboardEdges.add(e);
			}											 
		    }
		}
	    }
	}

	
    }

public void pasteSelection() {
	try {
		Object[] nodes = new Object[clipboardNodes];
		for (int i = 0; i < clipboardNodes; i++) {
			Vector node = windowManager.addNode();
			nodes[i] = (String) (node.elementAt(0));
			if (!((String) nodes[i]).equals(""))
				wfPersistence.addNode((String) nodes[i], (String) (node.elementAt(1)));
			else
				return;

			//	    wfGraph.addNewNode((String)nodes[i]);
		}

		for (Enumeration e = clipboardEdges.elements(); e.hasMoreElements();) {
			Edge edge = (Edge) e.nextElement();
			wfPersistence.addEdge((String) nodes[edge.source], (String) nodes[edge.target]);
			// wfGraph.restoreEdges();		
		}
	} catch (Exception e1) {
		JOptionPane.showMessageDialog(null, "System Error in paste selection");
	}

}

     public void newProject(){
        String temp;
	if((temp=windowManager.newProjectWindow())!=null){
	    /*if(activeProject!=null){
		JMSEvents jmse = new JMSEvents();
		jmse.sendProjectEvent(PROJECTCLOSE, activeProject);
	    }*/	    
	    activeProject=temp;
	    wfGraph.newGraph();
	    makeJMSConnection(activeProject);
	}
	updateStatus();
    }
    
    public void cloneProject(){
	windowManager.cloneProjectWindow();		
    }

    public void openProject(){
	windowManager.openProject();
	updateStatus();
    }

public void openProject(String project) {
	try {
		/*if (activeProject != null) {
			JMSEvents jmse = new JMSEvents();
			jmse.sendProjectEvent(PROJECTCLOSE, activeProject);
		}*/
		wfPersistence.openProject(project);
		wfGraph.openGraph(project);
		makeJMSConnection(project);
		activeProject = project;
		setTitle(project);
		setLayout1();
		updateStatus();
	} catch (Exception e1) {
		JOptionPane.showMessageDialog(null, "System Error in openProject");
		e1.printStackTrace();
	}

}
    public void setLayout1(){
	wfGraph.setLayout1();
    }
    public void setLayout2(){
	wfGraph.setExpand();
    }
   
    public void automaticLayout(){
	if(automatic){
	    automatic=false;
	    wfGraph.setAutomaticLayout(false);
	}else{
	    automatic=true;
	    wfGraph.setAutomaticLayout(true);
	}
    }
   
    public void setStatusBar(String info){
	frame.setStatusBar(info);
    }
    
    public Color getGraphBackground(){
	return wfGraph.getBackground();
    }

    public void setGraphBackground(Color color){
	wfGraph.setBackground(color);
    }

    public void zoom(int value){
	wfGraph.setZoom(value);
    }

    public void zoomIn(){
	wfGraph.zoomIn();
    }
    public void zoomOut(){
	wfGraph.zoomOut();
    }
    public void fitWindow(){
	wfGraph.fitWindow(frame.getScrollPane());
    }

    public BufferedImage getGraphImage(){
	return WFGraph.toImage(wfGraph);
    }

    public String getGraphVRML(){
	return WFGraph.vrmlView(wfGraph);
    }

    public WFGraph getGraph(){
	return wfGraph;
    }

    public JFrame getFrame(){
	return frame;
    }
    public void setTitle(String title){
	frame.setTitle("Workflow graph editor - BnProject : "+title+"  - BnUser: "+activeUser);
    }

    public WFPersistence getPersistence(){
	return wfPersistence;
    }

    public void changeRole (String nodeName){
	windowManager.changeRoleWindow(nodeName);
    }

    public void newUser (){
	windowManager.newUser();
    }
    public void newRole (){
	windowManager.newRole();
    }
    public void addUserToRole (){
	windowManager.addUserToRole();
    }
    public void usersInProject (){
	windowManager.usersInProject();
    }

    public void updateStatus() {
	try {
		String s = " ";
		String state;

		if (wfGraph.getSelectionNodes().length == 1 && wfGraph.getRoots().length > 0) {
			Object[] cell = wfGraph.getSelectionNodes();
			StrutsNodeValue nl = wfPersistence.getProjectNode((String) cell[0]);				
			String name = (String) cell[0];
			if (wfPersistence.isActive(name))
				s = name + " is " + nl.getState() + " by user " + nl.getRole();
			else
				s = name + " is " + nl.getState();

		} else {
			s = wfGraph.getVertices(wfGraph.getRoots()).length + " cell(s)";
		}
		frame.setStatusBar(s);

	} catch (Exception e1) {
		JOptionPane.showMessageDialog(null, "System Error in update status");
	}

}   
 

    

}// WFManager
