package hero.client.grapheditor;

/**
 * WFMenuBar.java
 *
 *
 * Created: Fri Aug 09 16:01:06 2002
 *
 * @version 1.0
 */

import java.awt.Color;
import java.awt.FileDialog;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JColorChooser;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;

public class WFMenuBar extends javax.swing.JMenuBar {
	Frame frame;

	public WFMenuBar(final Frame frame, final WFManager manager) {
		this.frame = frame;
		boolean buttonState = manager.getPersistence().hasPermission();
		ClassLoader cl = frame.getClass().getClassLoader();

		JMenu menu;
		JMenuItem mi;

		menu = new JMenu("File"); // File
		add(menu);

		// New
		mi = new JMenuItem(new ImageIcon(cl.getResource("images/Image1.gif")));
		mi.setText("New");
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.ALT_MASK));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.newProject();
			}
		});
		menu.add(mi);

		// Clone
		mi = new JMenuItem(new ImageIcon(cl.getResource("images/Clone.gif")));
		mi.setText("Clone");
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.ALT_MASK));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.cloneProject();
			}
		});
		menu.add(mi);

		// Open
		mi = new JMenuItem(new ImageIcon(cl.getResource("images/Open.gif")));
		mi.setText("Open");
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.ALT_MASK));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.openProject();
			}
		});
		menu.add(mi);

		menu.addSeparator();

		JMenu export = new JMenu("Export");
		export.setIcon(new ImageIcon(cl.getResource("images/Export.gif")));
		// ImageIO Writers
		AbstractAction a;

		String[] s = javax.imageio.ImageIO.getWriterFormatNames();
		for (int i = 0; i < s.length; i++) {
			final String format = s[i];
			if (format.toUpperCase().equals(format)) {
				a = new AbstractAction(format) {
					public void actionPerformed(ActionEvent e) {
						String name =
							saveDialog("Save as " + format.toUpperCase());
						if (manager.getGraph().getModel().getRootCount() > 0) {
							if (name != null) {
								try {
									BufferedImage im = manager.getGraphImage();
									if (im != null) {
										File f = new File(name);
										ImageIO.write(im, format, f);
										JOptionPane.showMessageDialog(
											null,
											"Exportation successful");
									} else
										error("");
								} catch (IOException ex) {
									error(ex.toString());
								}
							}
						} else
							error("");
					}
				};
				export.add(new JMenuItem(a));
			}
		}

		menu.addSeparator();

		AbstractAction b = new AbstractAction("VRML") {
			public void actionPerformed(ActionEvent e) {
				String name = saveDialog("Save as VRML");
				if (manager.getGraph().getModel().getRootCount() > 0) {
					if (name != null) {
						try {
							String data = manager.getGraphVRML();
							if (data != null) {
								File f = new File(name + ".wrl");
								FileOutputStream fout = new FileOutputStream(f);
								fout.write(data.getBytes());
								fout.close();
								JOptionPane.showMessageDialog(
									null,
									"Exportation succesful");
							} else
								error("");
						} catch (IOException ex) {
							error(ex.toString());
						}
					}
				} else
					error("");
			}
		};

		export.add(new JMenuItem(b));
		menu.add(export);

		menu.addSeparator();

		
		mi = new JMenuItem("Exit"); // - Exit
		mi.setIcon(new ImageIcon(cl.getResource("images/Stop.gif")));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.exit();
			}
		});
		menu.add(mi);

		menu = new JMenu("Edit"); //Edit
		add(menu);

		mi = new JMenuItem("Copy"); // - Copy	
		//	mi.setAccelerator(KeyStroke.getKeyStroke( KeyEvent.VK_C, ActionEvent.CTRL_MASK));
		mi.setIcon(new ImageIcon(cl.getResource("images/Copy.gif")));
		mi.setEnabled(buttonState);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.copySelection();
			}
		});
		menu.add(mi);
		mi = new JMenuItem("Paste"); // - Paste
		//	mi.setAccelerator(KeyStroke.getKeyStroke( KeyEvent.VK_V, ActionEvent.CTRL_MASK));
		mi.setIcon(new ImageIcon(cl.getResource("images/Paste.gif")));
		mi.setEnabled(buttonState);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.pasteSelection();
			}
		});
		menu.add(mi);

		mi = new JMenuItem("Delete"); // - Delete	
		mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));
		mi.setIcon(new ImageIcon(cl.getResource("images/Delete.gif")));
		mi.setEnabled(buttonState);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.deleteSelection();
			}
		});
		menu.add(mi);

		menu.addSeparator();

		mi = new JMenuItem("Select all", 'A'); // - Select all	
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.selectAll();
			}
		});
		menu.add(mi);

		mi = new JMenuItem("Deselect all", 'D'); // - Deselect all

		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.clearSelection();
			}
		});
		menu.add(mi);

		menu = new JMenu("Format"); //Format
		add(menu);
		mi = (JMenuItem) menu.add(new JMenuItem("Background"));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_B, ActionEvent.CTRL_MASK));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Color newColor =
					JColorChooser.showDialog(
						null,
						"Choose Background Color",
						manager.getGraphBackground());
				manager.setGraphBackground(newColor);

			}
		});
		menu.add(mi);

		final JCheckBoxMenuItem menuItem =
			new JCheckBoxMenuItem("Automatic Layout");
		menu.add(menuItem);
		menuItem.setState(true);
		menuItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.automaticLayout();
			}
		});
		menu.add(mi);

		menu = new JMenu("Users"); //Users
		menu.setEnabled(buttonState);
		add(menu);
		mi = (JMenuItem) menu.add(new JMenuItem("New BnUser"));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_U, ActionEvent.CTRL_MASK));
		mi.setIcon(new ImageIcon(cl.getResource("images/User.gif")));
		mi.setEnabled(buttonState);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.newUser();
			}
		});
		menu.add(mi);

		mi = (JMenuItem) menu.add(new JMenuItem("Users In BnProject"));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_T, ActionEvent.CTRL_MASK));
		mi.setIcon(new ImageIcon(cl.getResource("images/User2.gif")));
		mi.setEnabled(buttonState);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.usersInProject();
			}
		});
		menu.add(mi);

		menu.addSeparator();

		mi = (JMenuItem) menu.add(new JMenuItem("Add BnRole"));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK));
		mi.setIcon(new ImageIcon(cl.getResource("images/Role2.gif")));
		mi.setEnabled(buttonState);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.newRole();
			}
		});
		menu.add(mi);
		mi = (JMenuItem) menu.add(new JMenuItem("Add BnUser To BnRole"));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_J, ActionEvent.CTRL_MASK));
		mi.setIcon(new ImageIcon(cl.getResource("images/Role.gif")));
		mi.setEnabled(buttonState);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.addUserToRole();
			}
		});
		menu.add(mi);

		menu = new JMenu("Zoom"); //Zoom
		add(menu);

		mi = new JMenuItem("Fit Window");
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_F, ActionEvent.CTRL_MASK));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.fitWindow();
			}
		});
		menu.add(mi);

		mi = new JMenuItem("Zoom 100%");
		mi.setIcon(new ImageIcon(cl.getResource("images/Zoom100.gif")));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_M, ActionEvent.CTRL_MASK));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.zoom(100);
			}
		});
		menu.add(mi);

		mi = new JMenuItem("Zoom In");
		mi.setIcon(new ImageIcon(cl.getResource("images/ZoomIn.gif")));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_I, ActionEvent.CTRL_MASK));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.zoomIn();
			}
		});
		menu.add(mi);

		mi = new JMenuItem("Zoom Out");
		mi.setIcon(new ImageIcon(cl.getResource("images/ZoomOut.gif")));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.zoomOut();
			}
		});
		menu.add(mi);
		/*
			menu = new JMenu("Help");	    	//HELP
			add(menu);
		*/
	}

	/* Show a dialog with the given error message. */
	public void error(String message) {
		JOptionPane.showMessageDialog(
			this,
			message,
			"Info",
			JOptionPane.ERROR_MESSAGE);
	}
	/* Show a file save dialog and return the filename. */
	public String saveDialog(String message) {
		return dialog(message, FileDialog.SAVE);
	}

	/* Show a file load dialog and return the filename. */
	public String loadDialog(String message) {
		return dialog(message, FileDialog.LOAD);
	}

	/* Open a dialog and return the filename. Returns null if cancelled. */
	protected String dialog(String message, int mode) {
		FileDialog f = new FileDialog(frame, message, mode);
		f.show();
		if (f.getFile() != null)
			return f.getDirectory() + f.getFile();
		else
			return null;
	}

}
