package hero.client.test;

/*
*
* NodeTests.java - 
* Copyright (C) 2002 Ecoo Team
* charoy@loria.fr
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import hero.interfaces.BnNodeValue;
import hero.interfaces.BnProjectHome;
import hero.interfaces.BnUserHome;
import hero.interfaces.BnUserPropertyValue;
import hero.interfaces.Constants;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;
import hero.interfaces.UserSession;
import hero.interfaces.UserSessionHome;
import hero.interfaces.UserSessionUtil;
import hero.util.HeroException;

import java.util.Collection;
import java.util.Iterator;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.axis.client.Call;
import org.apache.axis.client.Service;

public class UserSessionTests extends TestCase {

    private BnProjectHome projecth;
    private ProjectSessionHome projectsesh;
    private BnUserHome userh;
    private UserSessionHome usersessionh;

    public UserSessionTests(String testname) {
	super(testname);
    }
    
    public static TestSuite suite() {
	return new TestSuite(UserSessionTests.class);
    }

    public void setUp() throws Exception {
    }

    public void testGetProjectList() throws Exception {
	projectsesh=ProjectSessionUtil.getHome();
	usersessionh=UserSessionUtil.getHome();
	UserSession usersession = usersessionh.create();
	usersession.init();
	ProjectSession lproj1 = projectsesh.create();
	lproj1.initProject("p1");
	ProjectSession lproj2 = projectsesh.create();
	lproj2.initProject("p2");		
	System.out.println("usersession home ?"+usersessionh);

	System.out.println("usersession ?"+usersession);
	Collection c = usersession.getProjectList();
	assertTrue("getProjectList Error",c.size() != 0);
    }

    /**
    * getToDoList test method in order to verify user ready activities
    */

    public void testGetToDoList1() throws Exception {
	projectsesh=ProjectSessionUtil.getHome();
	usersessionh=UserSessionUtil.getHome();
	ProjectSession proj = projectsesh.create();
	proj.initProject("p3");		
	
	proj.addNode("name1",Constants.Nd.AND_JOIN_NODE);
	proj.setUserRole("admin","InitialRole");

	UserSession usersession = usersessionh.create();
	Collection c = usersession.getToDoList("p3");
	Iterator i = c.iterator();
	assertTrue("getToDoList Error",((String)i.next()).equals("name1"));
    }

    public void testGetToDoList2() throws Exception {
	projectsesh=ProjectSessionUtil.getHome();
	usersessionh=UserSessionUtil.getHome();
	ProjectSession proj = projectsesh.create();
	proj.initProject("p4");		
	
	proj.addNode("name1",Constants.Nd.AND_JOIN_NODE);
	proj.addNode("name2",Constants.Nd.AND_JOIN_NODE);
	
	proj.setUserRole("admin","InitialRole");

	UserSession usersession = usersessionh.create();
	Collection c = usersession.getToDoList("p4");
	Iterator i = c.iterator();
	String node1 = (String)i.next();
	String node2 = (String)i.next();

	assertTrue("getToDoList Error",node1.equals("name1") || node1.equals("name2"));
	assertTrue("getToDoList Error",node2.equals("name1") || node2.equals("name2"));
    }

    /**
    * getActivityList test method in order to verify startActivity method
    * and obtain new executing users activities
    */

    public void testGetActivityList() throws Exception {
	projectsesh=ProjectSessionUtil.getHome();
	usersessionh=UserSessionUtil.getHome();
	ProjectSession proj = projectsesh.create();
	proj.initProject("p5");		
	
	proj.addNode("NodeExecuting",Constants.Nd.AND_JOIN_NODE);
	
	proj.setUserRole("admin","InitialRole");
	 
	UserSession usersession = usersessionh.create();
	Collection c=usersession.getActivityList("p5");
	assertTrue("getActivityList Error",c.size()==0);
	usersession.startActivity("p5","NodeExecuting");
	c = usersession.getActivityList("p5");
	Iterator i = c.iterator();
	assertTrue("getActivityList Error",((String)i.next()).equals("NodeExecuting"));
    }
    
    public void testGetNode() throws Exception {
	projectsesh=ProjectSessionUtil.getHome();
	usersessionh=UserSessionUtil.getHome();
	ProjectSession proj = projectsesh.create();
	proj.initProject("testgetnode");		
	proj.addNode("pouet",Constants.Nd.AND_JOIN_NODE);
	UserSession usr=usersessionh.create();
	try {
	    BnNodeValue nv=usr.getNode("testgetnode","pouet");
	} catch (HeroException h) {
	    assertTrue("Should be a success",false);
	}
    }
	
    public void testProperty() throws Exception {
	usersessionh=UserSessionUtil.getHome();
	UserSession usr=usersessionh.create();

	usr.setUserProperty("prop1","val1");
	usr.setUserProperty("prop2","val2");
	usr.setUserProperty("prop3","val3");
	
    }	
    public void testDeleteProperty() throws Exception {
	usersessionh=UserSessionUtil.getHome();
	UserSession usr=usersessionh.create();
	usr.deleteUserProperty("prop2");	
	usr.deleteUserProperty("prop3");	
    }	

    public void testGetProperties() throws Exception {
	usersessionh=UserSessionUtil.getHome();
	UserSession usr=usersessionh.create();
	
	Collection c=usr.getUserProperties();
        
	for(Iterator i=c.iterator();i.hasNext();){
	    BnUserPropertyValue p=(BnUserPropertyValue)i.next();
	    String key=p.getTheKey();
	    String value=p.getTheValue();
	    assertTrue("Get Properties",key.equalsIgnoreCase("prop1"));
	    
	}
    }


  public void testSpeedCheckOneCallRMI() throws Exception {
	usersessionh=UserSessionUtil.getHome();
	UserSession usr=usersessionh.create();
	usr.getUser();
	usr.getUserPassword();	
	
  }
  public void testSpeedCheckTenCallRMI() throws Exception {
	usersessionh=UserSessionUtil.getHome();
	UserSession usr=usersessionh.create();
	for(int i=0;i<10;i++){
	    usr.getUser();
	    usr.getUserPassword();	
	}
  }


  public void testSpeedCheckOneCallSOAP() throws Exception {
         //String endpoint = "http://localhost:8080/jboss-net/services/UserSession";
	 String endpoint =  System.getProperty("bonita.host")+"/bonita_ws/services/UserSession";
     
	 Service  service = new Service();
	 Call     call    = (Call) service.createCall();
	 
	 call.setTargetEndpointAddress( new java.net.URL(endpoint) );

	 call.setMaintainSession(true);
	 call.setUsername("admin");
	 call.setPassword("toto");

	 call.setOperationName("getUser");
	 String resp = (String) call.invoke( new Object[] {} );	           
	 call.setOperationName("getUserPassword");	 
	 String ret2 = (String) call.invoke( new Object[] {} );
	 
	 
  }

 public void testSpeedCheckTenCallSOAP() throws Exception {
    //     String endpoint = "http://localhost:8080/jboss-net/services/UserSession";
	String endpoint =  System.getProperty("bonita.host")+"/bonita_ws/services/UserSession";
     
	 Service  service = new Service();
	 Call     call    = (Call) service.createCall();
	 
	 call.setTargetEndpointAddress( new java.net.URL(endpoint) );

	 call.setMaintainSession(true);
	 call.setUsername("admin");
	 call.setPassword("toto");

	 for(int i=0;i<10;i++){
	     call.setOperationName("getUser");
	     String resp = (String) call.invoke( new Object[] {} );	           
	     call.setOperationName("getUserPassword");	 
	     String ret2 = (String) call.invoke( new Object[] {} );
	 } 
  }

}
