package hero.entity;

import hero.interfaces.BnAgentLocal;
import hero.interfaces.BnAgentLocalHome;
import hero.interfaces.BnEdgeLocal;
import hero.interfaces.BnEdgeLocalHome;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodeLocalHome;
import hero.interfaces.BnProjectLightValue;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectPK;
import hero.interfaces.BnProjectValue;
import hero.interfaces.InvalidValueException;
import hero.interfaces.SequenceGenerator;
import hero.interfaces.SequenceGeneratorHome;
import hero.interfaces.ServiceUnavailableException;
import hero.util.HeroException;

import java.rmi.RemoteException;
import java.util.Date;
import java.util.Collection;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import org.apache.log4j.Category;

/**
 * The Entity bean represents a BnProject
 *
 * @author Franois Charoy
 * @version $Revision: 0.1 $
 *
 * @ejb.bean name="BnProject"
 *           display-name="BnProject Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnProject"
 *           local-jndi-name="ejb/hero/BnProject_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"        
 *
 * @ejb.env-entry name="SequenceName"
 *                value="BnProject"
 *
 * @ejb.ejb-ref ejb-name="hero/SequenceGenerator"
 *
 * @ejb:transaction type="Supports"
 *
 * @ejb.permission unchecked="yes"
 * @ejb.pk generate="true" method-intf="remote"
 * 
 * @jonas.bean
 *     ejb-name="BnProject"
 *     jndi-name="ejb/hero/BnProject"
 * @jonas.jdbc-mapping
 *     jndi-name="jdbc_1"
 *     jdbc-table-name="BnProject"
 * @jonas.cleanup create
 *
 * @ejb.finder signature="java.util.Collection findAll()" 
 *     query="SELECT object(p) FROM BnProject p"
 *     transaction-type="Supports"
 *
 * @ejb.finder signature="hero.interfaces.BnProject findByName(java.lang.String pName )" result-type-mapping="Remote" 
 *     query="SELECT Object(p) From BnProject as p Where p.name = ?1"
 *     transaction-type="Supports"
 * @ejb.finder signature="hero.interfaces.BnProjectLocal findByName(java.lang.String pName )"  result-type-mapping="Local" 
 *     query="SELECT Object(p) From BnProject as p Where p.name = ?1"
 *     transaction-type="Supports"
 *
 *
 * @ejb.finder signature="java.util.Collection findByUser(java.lang.String userName )" 
 *     query="SELECT OBJECT(p) From BnProject as p, in (p.bnUsers) as u Where u.name = ?1"
 *     transaction-type="Supports"
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnProject"
 *
 * @ejb.value-object
 *    match="light"
 *    name="BnProjectLight"
 *
 * @jboss.table-name table-name="BnProject"
 *
 * @jboss.create-table create="true"
 *
 * @jboss.remove-table remove="false"
 *
 * @version $Revision: 1.1.1.1 $
 * @author $Author: charoy $
 **/
public abstract class BnProjectBean implements EntityBean {

	// -------------------------------------------------------------------------
	// Members
	// -------------------------------------------------------------------------   // -------------------------------------------------------------------------
	// Static
	// -------------------------------------------------------------------------
	// Utility variable
	private static final Category log =
		Category.getInstance(BnProjectBean.class);

	public EntityContext mContext;

	// -------------------------------------------------------------------------
	// Methods
	// -------------------------------------------------------------------------  

	/**
	* Retrive a unique creation id to use for this bean.  This will end up
	* demarcating this bean from others when it is stored as a record
	* in the database.
	*
	* @return Returns an integer that can be used as a unique creation id.
	*
	* @throws ServiceUnavailableException Indicating that it was not possible
	*                                     to retrieve a new unqiue ID because
	*                                     the service is not available
	**/
	private int generateUniqueId() throws ServiceUnavailableException {
		int lUniqueId = -1;
		try {
			Context lContext = new InitialContext();

			String lSequenceName =
				(String) lContext.lookup("java:comp/env/SequenceName");
			SequenceGeneratorHome lHome =
				(SequenceGeneratorHome) PortableRemoteObject.narrow(
					lContext.lookup("ejb/hero/SequenceGenerator"),
					SequenceGeneratorHome.class);
			SequenceGenerator lBean = (SequenceGenerator) lHome.create();
			lUniqueId = lBean.getNextNumber(lSequenceName);
			lBean.remove();
		} catch (NamingException ne) {
			throw new ServiceUnavailableException(
				"Naming lookup failure: " + ne.getMessage());
		} catch (CreateException ce) {
			throw new ServiceUnavailableException(
				"Failure while creating a generator session bean: "
					+ ce.getMessage());
		} catch (RemoveException re) {
			// When the Bean cannot be removed after a while it will be taken back by the container
			// therefore ignore this exception
		} catch (RemoteException rte) {
			throw new ServiceUnavailableException(
				"Remote exception occured while accessing generator session bean: "
					+ rte.getMessage());
		}

		return lUniqueId;
	}

	// -------------------------------------------------------------------------
	// Properties (Getters/Setters)
	// -------------------------------------------------------------------------  

	/**
	* Retrieve the BnProject's id.
	*
	* @return Returns an int representing the id of this BnUser.
	*
	* @ejb:pk-field
	* @ejb:interface-method view-type="both"
	* @ejb.value-object
	*      match="light"
	*
	* @ejb:persistence column-name="id"
	* @jonas.cmp-field-jdbc-mapping
	*    field-name="id"
	*    jdbc-field-name="id"
	* @ejb:transaction type="Supports"
	**/
	public abstract int getId();

	/**
	* Set the BnProject's id.
	*
	* @param pId The id of this BnUser. Is set at creation time.
	* @ejb:transaction type="Required"
	**/
	public abstract void setId(int pId);

	/**
	* Retrieve the BnProject's Name.
	*
	* @return Returns an int representing the Name of this BnProject.
	*
	* @ejb:interface-method view-type="both"
	* @ejb.value-object
	*      match="light"
	*
	* @ejb:persistence column-name="name"
	* @ejb:transaction type="Supports"
	**/
	public abstract String getName();

	/**
	* Set the BnProject's Name.
	*
	* @param pName The Name of this BnProject.  Is set at creation time.
	* @ejb:transaction type="Required"
	**/
	public abstract void setName(String pName);

	/**
	* Retrieve the BnProject Creator.
	*
	* @return Returns a string representing the name of the creator.
	*
	* @ejb.interface-method view-type="both"
	* @ejb.value-object
	*      match="light"
	*
	* @ejb:persistence column-name="creator"
	* @ejb:transaction type="Supports"
	**/
	public abstract String getCreator();

	/**
	* Set the Creator Name.
	*
	* @param pCreator The Creator of this project
	* @ejb:transaction type="Required"
	**/
	public abstract void setCreator(String pCreator);

	/**
	* @return Returns the State of this BnProject
	*
	* @ejb:interface-method view-type="both"
	* @ejb.value-object
	*      match="light"
	*
	* @ejb:persistence column-name="state"
	* @ejb:transaction type="Supports"
	**/
	public abstract int getState();

	/**
	* Specify the State of this BnProject
	*
	* @param pState State of this BnProject
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Required"
	**/
	public abstract void setState(int pState);

	/**
	 * @return Return the nodes of the project
	 * 
	 * @ejb.interface-method view-type="local"
	 * @ejb.relation name="Nodes" role-name="has-nodes"
	 *
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       compose="hero.interfaces.BnNodeValue"
	 *       compose-name="BnNode"
	 *       members="hero.interfaces.BnNodeLocal"
	 *       members-name="BnNode"
	 * @ejb:transaction type="Supports"
	 **/
	public abstract Collection getBnNodes();

	/**
	* Specify the Nodes of this BnProject
	*
	* @param pNodes Nodes of this BnProject
	* @ejb:interface-method view-type="local"
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnNodes(Collection pNodes);

	/**
	 * @return Return the edges of the project
	 * 
	 * @ejb:interface-method view-type="local"
	 * @ejb:relation name="Edges" role-name="has-edges"
	 *
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       compose="hero.interfaces.BnEdgeValue"
	 *       compose-name="BnEdge"
	 *       members="hero.interfaces.BnEdgeLocal"
	 *       members-name="BnEdge"     
	 * @ejb:transaction type="Supports"
	 **/
	public abstract Collection getBnEdges();

	/**
	* Specify the Edges of this BnProject
	*
	* @param pEdges Edges of this BnProject
	* @ejb:interface-method view-type="local"
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnEdges(Collection pEdges);

	/**
	 * @return Return the Agents of the project
	 * 
	 * @ejb.interface-method view-type="local"
	 * @ejb.relation name="Agents" role-name="has-agents"
	 *
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       compose="hero.interfaces.BnAgentValue"
	 *       compose-name="BnAgent"
	 *       members="hero.interfaces.BnAgentLocal"
	 *       members-name="BnAgent"
	 * @ejb:transaction type="Supports"
	 **/
	public abstract Collection getBnAgents();

	/**
	* Specify the Agents of this BnProject
	*
	* @param pAgents Agents of this BnProject
	* @ejb:interface-method view-type="local"
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnAgents(Collection pAgents);

	/**
	 * @return Return the agentedges of the project
	 * 
	 * @ejb:interface-method view-type="local"
	 * @ejb:relation name="AgentEdges" role-name="has-agentedges"
	 *
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       compose="hero.interfaces.BnAgentEdgeValue"
	 *       compose-name="BnAgentEdge"
	 *       members="hero.interfaces.BnAgentEdgeLocal"
	 *       members-name="BnAgentEdge"     
	 * @ejb:transaction type="Supports"
	 **/
	public abstract Collection getBnAgentEdges();

	/**
	* Specify the AgentEdges of this BnProject
	*
	* @param pAgentEdges AgentEdges of this BnProject
	* @ejb:interface-method view-type="local"
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnAgentEdges(Collection pAgentEdges);

	/**
	 * @return Return the users of the project
	 * 
	 * @ejb:interface-method view-type="local"
	 * @ejb:relation name="Users" role-name="has-users" target-multiple="yes"
	 * @jboss:auto-key-fields 
	 * @jonas :automatic-pk="true"
	 * @jboss.relation-table create-table="true" remove-table="false"
	 *
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       aggregate="hero.interfaces.BnUserLightValue"
	 *       aggregate-name="BnUser"
	 *       members="hero.interfaces.BnUserLocal"
	 *       members-name="BnUser"
	 * @ejb:transaction type="Supports"     
	 **/
	public abstract Collection getBnUsers();

	/**
	* Specify the Users of this BnProject
	*
	* @param pUsers Users of this BnProject
	*
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnUsers(Collection pUsers);

	/**
	 * @return Return the roles of the project
	 * 
	 * @ejb:interface-method view-type="local"
	 * @ejb:relation name="ProjectRoles" role-name="has-roles" target-multiple="yes"
	 * @jboss:auto-key-fields 
	 * @jonas :automatic-pk="true"
	 * @jboss.relation-table create-table="true" remove-table="false"
	 *
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       aggregate="hero.interfaces.BnRoleValue"
	 *       aggregate-name="BnRole"
	 *       members="hero.interfaces.BnRoleLocal"
	 *       members-name="BnRole"     
	 * @ejb:transaction type="Supports"
	 **/
	public abstract Collection getBnRoles();

	/**
	* Specify the Roles of this BnProject
	*
	* @param pRoles Roles of this BnProject
	*
	* @ejb:interface-method view-type="local"
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnRoles(Collection pRoles);

	/**
	 * @return Return the creation date of the BnProject
	 * @ejb:interface-method view-type="both"
	 * @ejb:persistence column-name="creationdate"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public abstract Date getCreationDate();

	/**
	 * @param pDate the creation date of the BnProject
	 * @ejb:interface-method view-type="both"
	 * @ejb.value-object
	 *      match="light"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public abstract void setCreationDate(Date pDate);

	/**
	 * @return Return the modification date of the BnProject
	 * @ejb:interface-method view-type="both"
	 * @ejb:persistence column-name="modificationdate"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public abstract Date getModificationDate();

	/**
	 * @param pDate the creation date of the BnProject
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public abstract void setModificationDate(Date pDate);

	/**
	 * @param nodeName name of the node to return
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public BnNodeLocal getBnNode(String nodeName) throws HeroException {
		BnNodeLocalHome nodehome;
		BnNodeLocal node;
		try {
			nodehome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			node = nodehome.findByName(nodeName, this.getId());
		} catch (FinderException fe) {
			throw new HeroException("No BnNode with this name");
		}

		return node;
	}

	/**
	 * @param AgentName name of the node to return
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public BnAgentLocal getBnAgent(String agentName) throws HeroException {
		BnAgentLocalHome agenthome;
		BnAgentLocal agent;
		try {
			agenthome = hero.interfaces.BnAgentUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			agent = agenthome.findByName(agentName, this.getId());
		} catch (FinderException fe) {
			throw new HeroException("No BnAgent with this name");
		}
		return agent;
	}

	/**
	 * @param edgeName name of the node to return
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public BnEdgeLocal getBnEdge(String edgeName) throws HeroException {
		BnEdgeLocalHome edgehome;
		BnEdgeLocal edge;
		try {
			edgehome = hero.interfaces.BnEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			edge = edgehome.findByName(edgeName, this.getId());
		} catch (FinderException fe) {
			throw new HeroException("No BnEdge with this name");
		}

		return edge;
	}

	/**
	 * @return  the properties of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:relation name="projectprp" role-name="is-property-of" 
	 * @jboss:auto-key-fields 
	 * @jonas :automatic-pk="true"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public abstract Collection getBnProperties();

	/**
	* Set the properties of the project
	* @ejb:interface-method view-type="both"
	* @param pPrp the properties of the project
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnProperties(Collection pPrp);

	/**
	 * @return  the Hooks of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:relation name="projecthook" role-name="is-hook-of" 
	 * @jboss:auto-key-fields 
	 * @jonas :automatic-pk="true"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public abstract Collection getBnHooks();

	/**
	* Set the Hooks of the project
	* @ejb:interface-method view-type="both"
	* @param pHooks the Hooks of the project
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnHooks(Collection pHooks);

	/**
	 * @return  the Interactives Hooks of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:relation name="projectinterhook" role-name="is-hook-of" 
	 * @jboss:auto-key-fields 
	 * @jonas :automatic-pk="true"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public abstract Collection getBnInterHooks();

	/**
	* Set the Interactives Hooks of the project
	* @ejb:interface-method view-type="both"
	* @param pHooks the Hooks of the project
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnInterHooks(Collection pHooks);

	/**
	 * @ejb.interface-method 
	 * @ejb:transaction type="Supports"
	 */
	public abstract BnProjectValue getBnProjectValue();

	/**
	 * @returns the light value of the bean (without relationships)
	 * @ejb.interface-method view-type=both
	 */
	public abstract BnProjectLightValue getBnProjectLightValue();

	/**
	 * @ejb.interface-method 
	 * @ejb:transaction type="Required"
	 */
	public abstract void setBnProjectValue(BnProjectValue v);



	// -------------------------------------------------------------------------
	// Framework Callbacks
	// -------------------------------------------------------------------------  

	/**
	* Create a BnProject based on the supplied BnProject Value Object.
	*
	* @throws InvalidValueException If one of the values are not correct,
	*                               this will not roll back the transaction
	*                               because the caller has the chance to
	*                               fix the problem and try again
	* @throws EJBException If no new unique ID could be retrieved this will
	*                      rollback the transaction because there is no
	*                      hope to try again
	* @throws CreateException Because we have to do so (EJB spec.)
	*
	* @ejb:create-method view-type="both"
	* @ejb:transaction type="Required"
	**/
	public BnProjectPK ejbCreate()
		throws InvalidValueException, EJBException, CreateException {
		try {
			// Each title must have a unique id to identify itself within the DB
			this.setId(generateUniqueId());
		} catch (ServiceUnavailableException se) {
			// The unique id could not be set therefore terminate the transaction
			// by throwing a system exception
			throw new EJBException(se.getMessage());
		}

		// This is only possible in CMPs. Otherwise return a valid PK.
		return null;
	}

	/**
	* Create a BnProject based on project name and creator.
	*
	* @param pName the name of the project
	* @param pCreator the creator of the project
	*
	* @throws EJBException If no new unique ID could be retrieved this will
	*                      rollback the transaction because there is no
	*                      hope to try again
	* @throws CreateException Because we have to do so (EJB spec.)
	*
	* @ejb:create-method view-type="both"
	* @ejb:transaction type="Required"
	**/
	public BnProjectPK ejbCreate(String pName, String creator)
		throws InvalidValueException, EJBException, CreateException {
		log.debug("ejbCreate: pName=" + pName + " creator=" + creator);
		BnProjectLocalHome projecthome;
		BnProjectLocal project;

		try {
			projecthome = hero.interfaces.BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new CreateException(be.getMessage());
		}
		try {
			project = projecthome.findByName(pName);
			throw new CreateException("BnProject " + pName + " already exist");
		} catch (FinderException fe) {

			try {
				// Each title must have a unique id to identify itself within the DB
				this.setId(generateUniqueId());
			} catch (ServiceUnavailableException se) {
				// The unique id could not be set therefore terminate the transaction
				// by throwing a system exception
				throw new EJBException(se.getMessage());
			}
			this.setName(pName);
			this.setCreator(creator);
		}

		// This is only possible in CMPs. Otherwise return a valid PK.
		return null;
	}

	public void setEntityContext(EntityContext lContext) {
		mContext = lContext;
	}

	public void unsetEntityContext() {
		mContext = null;
	}

	public void ejbActivate() {
	}

	public void ejbPassivate() {
	}

	public void ejbLoad() {
	}

	public void ejbStore() {
	}

	public void ejbRemove() throws RemoveException {
	}
}
