package hero.session;
/*
* 02/01/2002 - 15:24:07
*
* ProjectSessionEJB.java - 
* Copyright (C) 2002 Ecoo Team
* charoy@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
import hero.interfaces.BnAgentEdgeLocal;
import hero.interfaces.BnAgentLocal;
import hero.interfaces.BnEdgeLocal;
import hero.interfaces.BnNodeHookLocal;
import hero.interfaces.BnNodeInterHookLocal;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodePropertyLocal;
import hero.interfaces.BnProjectHookLocal;
import hero.interfaces.BnProjectInterHookLocal;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectUtil;
import hero.interfaces.BnProjectValue;
import hero.interfaces.ProjectSessionLocal;
import hero.interfaces.ProjectSessionLocalHome;
import hero.interfaces.ProjectSessionUtil;
import hero.util.EngineException;
import hero.util.EventConstants;
import hero.util.HeroException;
import hero.util.HeroHookException;
import hero.util.JMSEvents;

import java.util.Collection;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Category;

import bsh.Interpreter;

/**
 * Session Bean Template
 *
 * @ejb:bean name="Engine"
 *           display-name="Engine Bean"
 *           type="Stateful"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/Engine"
 *           local-jndi-name="ejb/hero/Engine_L"
 *
 * @ejb:ejb-ref ejb-name="BnProject"
 *              ref-name="myhero/BnProject"
 * @ejb:transaction type="Required"
 * @ejb:transaction-type type="Container"
 * @ejb.permission role-name="Admin,user"
 * @jonas.bean
 *      ejb-name="Engine"
 *      jndi-name="ejb/hero/Engine"
 **/

public class EngineBean implements SessionBean {

	// -------------------------------------------------------------------------
	// Static
	// -------------------------------------------------------------------------
	// Utility variable
	private static final Category log = Category.getInstance(EngineBean.class);

	// -------------------------------------------------------------------------
	// Members 
	// -------------------------------------------------------------------------

	private SessionContext mContext;

	private BnProjectLocalHome pHome;
	private BnProjectLocal mProject;
	private ProjectSessionLocalHome psHome;
	private ProjectSessionLocal mPSession;
	private JMSEvents jms;

	// -------------------------------------------------------------------------
	// Methods
	// -------------------------------------------------------------------------  
	/**
	 * Terminates the project
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void terminate() throws EngineException, HeroException {
		log.debug("terminate:");
		if (nodesTerminated(mProject.getBnNodes())) {
			mProject.setState(hero.interfaces.Constants.Pj.TERMINATED);
		} else {
			throw new EngineException("BnProject cannot be terminated. Some nodes are still active");
		}
	}

	/**
	 * Starts the activity nodeName
	 * @param nodeName the name of the activity
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void startActivity(String nodeName)
		throws EngineException, HeroException {
		log.debug("startActivity: nodeName=" + nodeName);
		execute(nodeName, hero.interfaces.Constants.Nd.START);
	}

	/**
	 * Starts the activity nodeName
	 * @param nodeName the name of the activity
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void terminateActivity(String nodeName)
		throws EngineException, HeroException {
		log.debug("terminateActivity: nodeName=" + nodeName);
		execute(nodeName, hero.interfaces.Constants.Nd.TERMINATE);
	}

	/**
	 * Starts the activity nodeName
	 * @param nodeName the name of the activity
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void cancelActivity(String nodeName)
		throws EngineException, HeroException {
		log.debug("cancelActivity: nodeName=" + nodeName);
		execute(nodeName, hero.interfaces.Constants.Nd.CANCEL);
	}

	/**
	 * Suspend the activity nodeName
	 * @param nodeName the name of the activity
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void suspendActivity(String nodeName)
		throws EngineException, HeroException {
		log.debug("suspendActivity: nodeName=" + nodeName);
		execute(nodeName, hero.interfaces.Constants.Nd.SUSPEND);
	}

	/**
	 * Non_Automatic operation method that resume specify activity
	 */

	/**
	 * Resume the activity nodeName
	 * @param nodeName the name of the activity
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void resumeActivity(String nodeName)
		throws EngineException, HeroException {
		log.debug("resumeActivity: nodeName=" + nodeName);
		execute(nodeName, hero.interfaces.Constants.Nd.RESUME);
	}

	/**
	 * Active an External BnAgent
	 * @param agentName the name of the external agent
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void activeAgent(String agentName)
		throws EngineException, HeroException {
		log.debug("activeAgent: agentName=" + agentName);
		BnAgentLocal ag;
		ag = mProject.getBnAgent(agentName);
		mPSession.setAgentState(ag, hero.interfaces.Constants.Ag.ACTIVE);
		Collection edges = ag.getOutEdges();
		if (!edges.isEmpty())
			modifyAgentEdges(edges);
	}

	private void execute(String _node, int operation)
		throws HeroException, EngineException {
		log.debug(
			"execute: node="
				+ _node
				+ " operation="
				+ hero.interfaces.Constants.Nd.operationName[operation]);
		BnNodeLocal nd;
		nd = mProject.getBnNode(_node);

		int newState = nd.getTransition().computeState(nd, operation);
		log.debug(
			"Execute: New node State "
				+ _node
				+ " :"
				+ hero.interfaces.Constants.Nd.nodeStateName[newState]);

		if (newState == hero.interfaces.Constants.Nd.BAD_TRANSITION)
			throw new EngineException("Bad state transition " + _node);

		if (newState != nd.getState()) {
			if (newState == hero.interfaces.Constants.Nd.TERMINATED) {
				jms.sendProjectEvent(
					EventConstants.TERMINATED,
					mProject.getName(),
					_node,
					mContext.getCallerPrincipal().getName());

				this.hookExecute(_node, hero.interfaces.Constants.Nd.EXECUTE);
				this.hookExecute(
					_node,
					hero.interfaces.Constants.Nd.BEFORETERMINATE);
				mPSession.propagateNodeProperties(_node);
			} else if (newState == hero.interfaces.Constants.Nd.EXECUTING) {
				jms.sendProjectEvent(
					EventConstants.START,
					mProject.getName(),
					_node,
					mContext.getCallerPrincipal().getName());
				this.hookExecute(
					_node,
					hero.interfaces.Constants.Nd.BEFORESTART);
			} else if (newState == hero.interfaces.Constants.Nd.ANTICIPATING) {
				jms.sendProjectEvent(
					EventConstants.ANTICIPATING,
					mProject.getName(),
					_node,
					mContext.getCallerPrincipal().getName());
				this.hookExecute(
					_node,
					hero.interfaces.Constants.Nd.ANTICIPATE);
			} else if (newState == hero.interfaces.Constants.Nd.DEAD) {
				jms.sendProjectEvent(
					EventConstants.CANCELED,
					mProject.getName(),
					_node,
					mContext.getCallerPrincipal().getName());
			}

			if (activityStarted(newState)) {
				this.hookExecute(
					_node,
					hero.interfaces.Constants.Nd.AFTERSTART);
			}
			if (newState == hero.interfaces.Constants.Nd.TERMINATED) {
				this.hookExecute(
					_node,
					hero.interfaces.Constants.Nd.AFTERTERMINATE);
			}
			mPSession.setNodeState(nd, newState);
			mPSession.setNodeExecutor(_node);
			Collection edges = nd.getOutBnEdges();
			if (!edges.isEmpty())
				modifyNodeEdges(newState, edges);

		}
	}

	/**
	 * Method that consult new node state value and modify out edges (state 
	 * propagation) 
	 */

	private void modifyNodeEdges(int nodeState, Collection edges)
		throws HeroException, EngineException {
		int newEdgeState;
		Iterator i = edges.iterator();
		//	newEdgeState = hero.interfaces.Constants.edgeTransition[evaluateCondition(edge)][nodeState];
		while (i.hasNext()) {
			BnEdgeLocal edge = (BnEdgeLocal) i.next();
			if (nodeState != hero.interfaces.Constants.Nd.TERMINATED) {
				newEdgeState =
					hero
						.interfaces
						.Constants
						.edgeTransition[hero
						.interfaces
						.Constants
						.Ed
						.CONDITION_TRUE][nodeState];
				mPSession.setEdgeState(edge, newEdgeState);
				edgeChanged(edge);
			} else {
				newEdgeState =
					hero.interfaces.Constants.edgeTransition[evaluateCondition(
						edge)][nodeState];
				if (edge.getState() != newEdgeState) {
					mPSession.setEdgeState(edge, newEdgeState);
					edgeChanged(edge);
				}
			}
		}
	}

	/**
	 * Method that propagate the AgentEdges state
	 */

	private void modifyAgentEdges(Collection edges)
		throws HeroException, EngineException {
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnAgentEdgeLocal edge = (BnAgentEdgeLocal) i.next();
			mPSession.setAgentEdgeState(
				edge,
				hero.interfaces.Constants.Ed.ANTICIPATING);
			BnNodeLocal nd = edge.getOutBnNode();
			int operation = nd.getActivation().state(nd);
			execute(nd.getName(), hero.interfaces.Constants.Nd.ANTACTIVE);
		}
	}

	private void edgeChanged(BnEdgeLocal edge)
		throws HeroException, EngineException {
		BnNodeLocal nd = edge.getOutBnNode();
		if (nd.getState() != hero.interfaces.Constants.Nd.DEAD) {
			int operation = nd.getActivation().state(nd);
			execute(nd.getName(), operation);
		}
	}

	// Utility methods
	private boolean activityStarted(int state) throws HeroException {
		return (
			state == hero.interfaces.Constants.Nd.ANTICIPATING
				|| state == hero.interfaces.Constants.Nd.EXECUTING);
	}

	/*private boolean activityStarted(BnNodeLocal nd) throws HeroException {
		int state = nd.getState();
		return (
			state == hero.interfaces.Constants.Nd.ANTICIPATING
				|| state == hero.interfaces.Constants.Nd.EXECUTING);
	}*/

	private void hookExecute(String nodeName, String eventName)
		throws HeroException, HeroHookException {
		Collection hooks;
		Collection interHooks;
		BnNodeLocal node;
		log.debug("hookExecute " + nodeName + " " + eventName);

		node = mProject.getBnNode(nodeName);
		hooks = node.getBnHooks();
		for (Iterator i = hooks.iterator(); i.hasNext();) {
			BnNodeHookLocal hk = (BnNodeHookLocal) i.next();
			if (hk.getEvent().equalsIgnoreCase(eventName)) {
				hero.hook.Hook h =
					hero.hook.Hook.make(hk.getName(), eventName, hk.getType());
				h.execute(this, eventName, node);
			}
		}

		interHooks = node.getBnInterHooks();
		for (Iterator i = interHooks.iterator(); i.hasNext();) {
			BnNodeInterHookLocal hk = (BnNodeInterHookLocal) i.next();
			if (hk.getEvent().equalsIgnoreCase(eventName)) {
				hero.hook.Hook h =
					hero.hook.Hook.make(hk.getName(), eventName, hk.getType());
				h.execute(this, eventName, node);
			}
		}

		hooks = mProject.getBnHooks();
		for (Iterator i = hooks.iterator(); i.hasNext();) {
			BnProjectHookLocal hk = (BnProjectHookLocal) i.next();
			if (hk.getEvent().equalsIgnoreCase(eventName)) {
				hero.hook.Hook h =
					hero.hook.Hook.make(hk.getName(), eventName, hk.getType());
				h.execute(this, eventName, node);
			}
		}

		interHooks = mProject.getBnInterHooks();
		for (Iterator i = interHooks.iterator(); i.hasNext();) {
			BnProjectInterHookLocal hk = (BnProjectInterHookLocal) i.next();
			if (hk.getEvent().equalsIgnoreCase(eventName)) {
				hero.hook.Hook h =
					hero.hook.Hook.make(hk.getName(), eventName, hk.getType());
				h.execute(this, eventName, node);
			}
		}

	}

	private boolean nodesTerminated(Collection nodes) {
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			BnNodeLocal node = (BnNodeLocal) i.next();
			int state = node.getState();
			if (state != hero.interfaces.Constants.Nd.TERMINATED
				&& state != hero.interfaces.Constants.Nd.EXPIRED
				&& state != hero.interfaces.Constants.Nd.DEAD)
				return false;
		}
		return true;
	}

	public int evaluateCondition(BnEdgeLocal e) {
		try {
			String condition = e.getCondition();
			if (condition != null) {
				condition =
					"import hero.interfaces.BnEdgeLocal;\n"
						+ "import hero.interfaces.BnNodeLocal;\n"
						+ "boolean evalCondition (Object edge) {\n\n\n"
						+ "return("
						+ condition
						+ ");"
						+ "}";
				Interpreter i = new Interpreter();

				// BnNode properties mapping

				Collection prop =
					((BnNodeLocal) e.getInBnNode()).getBnProperties();
				for (Iterator props = prop.iterator(); props.hasNext();) {
					BnNodePropertyLocal pl = (BnNodePropertyLocal) props.next();
					i.set(pl.getTheKey(), pl.getTheValue());
				}

				i.set("edge", e);
				i.eval(condition);
				i.eval("result=evalCondition(edge)");
				if (((Boolean) i.get("result")).booleanValue() == true)
					return (hero.interfaces.Constants.Ed.CONDITION_TRUE);
				else
					return (hero.interfaces.Constants.Ed.CONDITION_FALSE);
			} else
				return (hero.interfaces.Constants.Ed.CONDITION_TRUE);
		} catch (Exception t) {
			t.printStackTrace();
			return (hero.interfaces.Constants.Ed.CONDITION_FALSE);
		}
	}

	/**
	 * init the Engine Session Bean
	 *
	 * @throws HeroException 
	 * @param projectName project name 
	 * @ejb:interface-method view-type="both"
	 **/
	public void initProject(String projectName) throws CreateException {
		log.debug("ejbCreate: project=" + projectName);
		try {
			pHome = BnProjectUtil.getLocalHome();
			psHome = ProjectSessionUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new CreateException(ne.getMessage());
		}
		try {
			mProject = pHome.findByName(projectName);
			mPSession = psHome.create();
			mPSession.initProject(projectName);
		} catch (FinderException fe) {
			throw new CreateException(
				"BnProject does not exist " + projectName);
		} catch (HeroException he) {
					throw new CreateException(
				"ProjectSession Error" + projectName);
		}		
	}

	/**
	 *
	 * @throws CreateException 
	 *
	 * @ejb:create-method view-type="both"
	 **/
	public void ejbCreate() throws CreateException {
	}

public void setSessionContext(final javax.ejb.SessionContext context) {
	mContext = context;
	jms = new JMSEvents();
}

public void ejbRemove() {
}

public void ejbActivate() {
}

public void ejbPassivate() {
}

}
