package hero.session;
/*
* 02/01/2002 - 15:24:07
*
* ProjectSessionEJB.java - 
* Copyright (C) 2002 Ecoo Team
* charoy@loria.fr
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import hero.entity.EdgeState;
import hero.entity.NodeState;
import hero.interfaces.BnAgentEdgeLocal;
import hero.interfaces.BnAgentEdgeLocalHome;
import hero.interfaces.BnAgentEdgeValue;
import hero.interfaces.BnAgentLocal;
import hero.interfaces.BnAgentLocalHome;
import hero.interfaces.BnAgentValue;
import hero.interfaces.BnEdgeLocal;
import hero.interfaces.BnEdgeLocalHome;
import hero.interfaces.BnEdgeUtil;
import hero.interfaces.BnEdgeValue;
import hero.interfaces.BnNodeHookLocal;
import hero.interfaces.BnNodeHookLocalHome;
import hero.interfaces.BnNodeHookUtil;
import hero.interfaces.BnNodeInterHookLocal;
import hero.interfaces.BnNodeInterHookLocalHome;
import hero.interfaces.BnNodeInterHookUtil;
import hero.interfaces.BnNodeLightValue;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodeLocalHome;
import hero.interfaces.BnNodePropertyLocal;
import hero.interfaces.BnNodePropertyLocalHome;
import hero.interfaces.BnNodePropertyUtil;
import hero.interfaces.BnNodeUtil;
import hero.interfaces.BnNodeValue;
import hero.interfaces.BnProjectHookLocal;
import hero.interfaces.BnProjectHookLocalHome;
import hero.interfaces.BnProjectHookUtil;
import hero.interfaces.BnProjectInterHookLocal;
import hero.interfaces.BnProjectInterHookLocalHome;
import hero.interfaces.BnProjectInterHookUtil;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectPropertyLocal;
import hero.interfaces.BnProjectPropertyLocalHome;
import hero.interfaces.BnProjectPropertyUtil;
import hero.interfaces.BnProjectValue;
import hero.interfaces.BnRoleLocal;
import hero.interfaces.BnRoleLocalHome;
import hero.interfaces.BnRoleUtil;
import hero.interfaces.BnRoleValue;
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.interfaces.BnUserUtil;
import hero.interfaces.InvalidValueException;
import hero.interfaces.Constants;
import hero.util.DeleteEdgeException;
import hero.util.EventConstants;
import hero.util.HeroException;
import hero.util.JMSEvents;
import hero.util.NodeCancelledException;
import hero.util.NodeExecutingException;
import hero.util.NodeTerminatedException;
import hero.util.StrutsEdgeValue;
import hero.util.StrutsNodeValue;
import hero.util.TimerFilter;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.NotificationFilter;
import javax.management.ObjectName;
import javax.naming.Context;

import org.apache.log4j.Category;

/**
 * Session Bean Template
 *
 * @ejb:bean name="ProjectSession"
 *           display-name="ProjectSession Bean"
 *           type="Stateful"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/ProjectSession"
 *           local-jndi-name="ejb/hero/ProjectSession_L"
 *
 * @ejb:ejb-ref	ejb-name="ProjectSession"
 *              	view-type="remote"
 *              	ref-name="ProjectSession"
 *
 * @ejb:ejb-ref ejb-name="BnProject"
 *              ref-name="myhero/BnProject"
 *
 * @ejb:transaction type="Supports"
 * @ejb.permission role-name="Admin,user"
 * @jonas.bean
 *      ejb-name="ProjectSession"
 *      jndi-name="ejb/hero/ProjectSession"
 *
 **/


public class ProjectSessionBean implements SessionBean, EventConstants {

	// -------------------------------------------------------------------------
	// Static
	// -------------------------------------------------------------------------
	// Utility variable
	private static final Category log = Category.getInstance(ProjectSessionBean.class);

	// -------------------------------------------------------------------------
	// Members 
	// -------------------------------------------------------------------------

	private SessionContext mContext;

	private BnProjectLocalHome pHome;
	private BnProjectLocal mProject;
	private JMSEvents jms;

	// -------------------------------------------------------------------------
	// Methods
	// -------------------------------------------------------------------------

	// -------------------------------------------------------------------------
	// Add business methods
	// -------------------------------------------------------------------------

	/**
	 * Add a BnUser to the BnProject
	 *
	 * @param userName The name of the user to associate to the project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void addUser(String userName) throws HeroException {
		log.debug("addUser: name=" + userName);
		BnUserLocalHome uHome;
		BnRoleLocalHome rHome;
		BnUserLocal mUser;

		try {
			uHome = hero.interfaces.BnUserUtil.getLocalHome();
			rHome = hero.interfaces.BnRoleUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			mUser = uHome.findByName(userName);
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException("BnUser does not exist");
		}

		Collection cUsers = mProject.getBnUsers();
		cUsers.add(mUser);
		this.addRole(userName, "BnUser BnRole");
		this.setUserRole(userName, userName);
		jms.sendProjectEvent(ADDUSERPROJECT, this.getDetails().getName(), userName);

	}

	/**
	 * Add a BnNode to the BnProject
	 *
	 * @param name The name of the node to add to the project
	 * @param nodeType The type of the node to add to the project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/

	public void addNode(String name, int nodeType) throws HeroException {
		log.debug("addNode: name=" + name + " nodeType:");
		BnNodeLocalHome nHome;
		BnNodeLocal mNode;
		
		
		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}

		try {
			BnNodeLocal nf = nHome.findByName(name, mProject.getId());
			throw new HeroException("BnNode " + name + " already exist in project " + mProject.getName());
		} catch (javax.ejb.FinderException nn) {

			try {
				BnNodeLocal nd = nHome.create(name, nodeType);
				BnNodeValue ndata = nd.getBnNodeValue();
				ndata.setCreationDate(new java.util.Date());
				nd.setBnNodeValue(ndata);
				Collection cNodes = mProject.getBnNodes();
				cNodes.add(nd);
				String creator = mContext.getCallerPrincipal().getName();				
				BnUserLocalHome uHome = hero.interfaces.BnUserUtil.getLocalHome();
				BnUserLocal user = uHome.findByName(creator);
				nd.setCreator(user);
				nd.setExecutor(user);  // Only for consistency
				BnRoleLocalHome rHome = BnRoleUtil.getLocalHome();
				BnRoleLocal role = rHome.findByName(hero.interfaces.Constants.INITIALROLE, mProject.getId());
				nd.setBnRole(role);

				// Tell the world
				jms.sendNodeEvent(ADDNODE, this.getDetails().getName(), name, nodeType, ndata.getState(), mContext.getCallerPrincipal().getName());
			} catch (InvalidValueException ie) {
						throw new HeroException(ie.getMessage());
			} catch (CreateException ce) {
				throw new HeroException(ce.getMessage());
			} catch (FinderException fe) {
				throw new HeroException(fe.getMessage());
			} catch (javax.naming.NamingException ne) {
				throw new HeroException(ne.getMessage());
			}
		}
	}

	/**
	 * Add an BnAgent to the BnProject
	 *
	 * @param name The name of the BnAgent to add to the project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void addAgent(String name) throws HeroException {
		log.debug("addNode: name=");
		BnAgentLocalHome nHome;
		BnAgentLocal mAgent;
		try {
			nHome = hero.interfaces.BnAgentUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}

		try {
			BnAgentLocal nf = nHome.findByName(name, mProject.getId());
			throw new HeroException("BnAgent " + name + " already exist in project " + mProject.getName());
		} catch (javax.ejb.FinderException nn) {

			try {
				BnAgentLocal ag = nHome.create(name);
				BnAgentValue adata = ag.getBnAgentValue();
				adata.setCreationDate(new java.util.Date());
				ag.setBnAgentValue(adata);
				Collection cAgents = mProject.getBnAgents();
				cAgents.add(ag);
				String creator = mContext.getCallerPrincipal().getName();
				BnUserLocalHome uHome = hero.interfaces.BnUserUtil.getLocalHome();
				BnUserLocal user = uHome.findByName(creator);
				ag.setCreator(user);

				// Tell the world
				//		jms.sendNodeEvent(ADDNODE,this.getDetails().getName(),name,nodeType,ndata.getState(),mContext.getCallerPrincipal().getName());
			} catch (InvalidValueException ie) {
						throw new HeroException(ie.getMessage());
			} catch (CreateException ce) {
				throw new HeroException(ce.getMessage());
			} catch (FinderException fe) {
				throw new HeroException(fe.getMessage());
			} catch (javax.naming.NamingException ne) {
				throw new HeroException(ne.getMessage());
			}
		}
	}

	/**
	 * add an BnEdge to the BnProject
	 *
	 * @param name the name ot the edge
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void addEdge(String name, String in, String out) throws HeroException {
		log.debug("addEdge: name=" + name + " in=" + in + " out=" + out);
		throw new HeroException("no more implemented");
	}

	/**
	 * Add an BnEdge to the BnProject [in] ---- [out]
	 *
	 * @param in the name of the in BnNode
	 * @param out the name of the out BnNode
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public String addEdge(String in, String out) throws HeroException {
		log.debug("addEdge: in=" + in + " out:" + out);
		BnEdgeLocalHome eHome;
		BnNodeLocalHome nHome;
		BnNodeLocal mInNode;
		BnNodeLocal mOutNode;
		BnEdgeLocal mEdge;
		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			mInNode = nHome.findByName(in, mProject.getId());
		} catch (FinderException fe) {
			throw new HeroException("In BnNode does not exist");
		}
		try {
			mOutNode = nHome.findByName(out, mProject.getId());
		} catch (FinderException fe) {
			throw new HeroException("Out BnNode does not exist");
		}
		try {
			eHome = hero.interfaces.BnEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		if (checkDynamics(out)) { // check state of Nodes
			if (checkDuplicateEdge(in, out)) {
				try {
					mEdge = eHome.create();
					mEdge.setBnProject(mProject);
					mEdge.setInBnNode(mInNode);
					mEdge.setOutBnNode(mOutNode);

					jms.sendEdgeEvent(
						ADDEDGE,
						this.getDetails().getName(),
						mEdge.getName(),
						mInNode.getName(),
						mOutNode.getName(),
						mContext.getCallerPrincipal().getName());

				} catch (InvalidValueException ie) {
					throw new HeroException(ie.getMessage());
				} catch (CreateException ce) {
					throw new HeroException(ce.getMessage());
				}
			} else {
				throw new hero.util.DuplicatedEdgeException("Cannot add edge. BnEdge between nodes already exist");
			}
		} else {
			throw new hero.util.NodeStartedException("Cannot add edge. " + out + " is already started");
		}
		if (!checkStructuralConstraints())
			throw new hero.util.EdgeCycleException("This edge add a cycle");

		// update the current graph state
		int edgeState = hero.interfaces.Constants.edgeTransition[hero.interfaces.Constants.Ed.CONDITION_TRUE][mInNode.getState()];
		this.setEdgeState(mEdge, edgeState);
		//	mEdge.setState(edgeState);

		int newOperation = mOutNode.getActivation().state(mOutNode);
		int newState = mOutNode.getTransition().computeState(mOutNode, newOperation);
		if (newState == hero.interfaces.Constants.Nd.BAD_TRANSITION) {
			throw new HeroException("AddEdge generated a bad BnNode transition");
		}
		this.setNodeState(mOutNode, newState);
		//	jms.sendNodeEvent(READYINITIAL,this.getName(),nOut.getName(),nOut.getType(),INITIAL,context.getCallerPrincipal().getName());

		return mEdge.getName();
	}

	/**
	 * Add an BnAgentEdge to the BnProject [in] ---- [out]
	 *
	 * @param in the name of the in BnAgent
	 * @param out the name of the out BnNode
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public String addAgentEdge(String in, String out) throws HeroException {
		log.debug("addAgentEdge: in=" + in + " out:" + out);
		BnAgentEdgeLocalHome eHome;
		BnNodeLocalHome nHome;
		BnAgentLocalHome aHome;
		BnAgentLocal mInAgent;
		BnNodeLocal mOutNode;
		BnAgentEdgeLocal mEdge;
		try {
			aHome = hero.interfaces.BnAgentUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			mInAgent = aHome.findByName(in, mProject.getId());
		} catch (FinderException fe) {
			throw new HeroException("In BnNode does not exist");
		}
		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			mOutNode = nHome.findByName(out, mProject.getId());
		} catch (FinderException fe) {
			throw new HeroException("Out BnNode does not exist");
		}
		try {
			eHome = hero.interfaces.BnAgentEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		if (checkDynamics(out)) { // check state of Nodes
			if (checkDuplicateAgentEdge(in, out)) {
				try {
					mEdge = eHome.create();
					mEdge.setBnProject(mProject);
					mEdge.setInBnAgent(mInAgent);
					mEdge.setOutBnNode(mOutNode);

					//jms.sendEdgeEvent(ADDEDGE,this.getDetails().getName(),mEdge.getName(),mInNode.getName(),mOutNode.getName(),mContext.getCallerPrincipal().getName());

				} catch (InvalidValueException ie) {
					throw new HeroException(ie.getMessage());
				} catch (CreateException ce) {
					throw new HeroException(ce.getMessage());
				}
			} else {
				throw new hero.util.DuplicatedEdgeException("Cannot add agentEdge. BnAgentEdge between nodes already exist");
			}
		} else {
			throw new hero.util.NodeStartedException("Cannot add agentEdge. " + out + " is already started");
		}
		if (!checkStructuralConstraints())
			throw new hero.util.EdgeCycleException("This edge add a cycle");

		// update the current graph state
		this.setAgentEdgeState(mEdge, hero.interfaces.Constants.Ed.INITIAL);

		int newOperation = mOutNode.getActivation().state(mOutNode);
		int newState = mOutNode.getTransition().computeState(mOutNode, newOperation);
		if (newState == hero.interfaces.Constants.Nd.BAD_TRANSITION) {
			throw new HeroException("AddEdge generated a bad BnNode transition");
		}
		this.setNodeState(mOutNode, newState);
		//	jms.sendNodeEvent(READYINITIAL,this.getName(),nOut.getName(),nOut.getType(),INITIAL,context.getCallerPrincipal().getName());

		return mEdge.getName();
	}

	/**
	 * Add a role to the project
	 *
	 * @param name the name of the role
	 * @param description the description of the role
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void addRole(String roleName, String description) throws HeroException {
		BnRoleLocalHome roleh;
		log.debug("addRole: roleName=" + roleName + " description=" + description);
		try {
			roleh = BnRoleUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		BnRoleValue rv = new BnRoleValue();
		rv.setName(roleName);
		rv.setDescription(description);
		try {
			BnRoleLocal rl = roleh.create(rv);
			mProject.getBnRoles().add(rl);
			jms.sendRoleEvent(ADDROLE, this.getDetails().getName(), roleName);
		} catch (javax.ejb.CreateException ce) {
			throw new HeroException(ce.getMessage());
		}
	}

	/**
	 * add hook to project
	 *
	 * @param hookName the name of the hook
	 * @param eventName the event
	 * @param hookType the type
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void addHook(String hookName, String eventName, int hookType) throws HeroException {
		log.debug("addHook: hookName=" + hookName + " eventName=" + eventName);
		BnProjectHookLocalHome pHome;
		BnProjectHookLocal ppLocal;

		try {
			pHome = BnProjectHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}

		try {
			BnProjectHookLocal pf = pHome.findByName(mProject.getId(), hookName);
			throw new HeroException("BnProject hook " + hookName + " already exist in project " + mProject.getName());
		} catch (javax.ejb.FinderException pn) {

			try {
				BnProjectHookLocal h = pHome.create(hookName, eventName, hookType);
				h.setBnProject(mProject);
			} catch (CreateException fe) {
				throw new HeroException(fe.getMessage());
			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			}
		}
	}

	/**
	 * add hook to a node
	 *
	 * @param nodeName the name of the node
	 * @param hookName the name of the hook
	 * @param eventName the name of the event
	 * @param hookType the type of the hook
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void addNodeHook(String nodeName, String hookName, String eventName, int hookType) throws HeroException {
		log.debug("addNodeHook: node=" + nodeName + " hookName=" + hookName + " eventName=" + eventName);
		BnNodeHookLocalHome pHome;
		BnNodeHookLocal ppLocal;

		BnNodeLocal nd = mProject.getBnNode(nodeName);
		try {
			pHome = BnNodeHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}

		try {
			BnNodeHookLocal nf = pHome.findByName(nd.getId(), hookName);
			throw new HeroException("BnNode hook " + hookName + " already exist in node " + nodeName);
		} catch (javax.ejb.FinderException nn) {

			try {
				BnNodeHookLocal h = pHome.create(hookName, eventName, hookType);
				h.setBnNode(nd);
			} catch (CreateException fe) {
				throw new HeroException(fe.getMessage());
			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			}
		}
	}

	/**
	 * add interhook to a node
	 *
	 * @param nodeName the name of the node
	 * @param hookName the name of the hook
	 * @param eventName the name of the event
	 * @param hookType the type of the hook
	 * @param script the value of the script 
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void addNodeInterHook(String nodeName, String hookName, String eventName, int hookType, String value) throws HeroException {
		log.debug("addNodeHook: node=" + nodeName + " hookName=" + hookName + " eventName=" + eventName + " value=" + value);
		BnNodeInterHookLocalHome pHome;
		BnNodeInterHookLocal ppLocal;

		BnNodeLocal nd = mProject.getBnNode(nodeName);
		try {
			pHome = BnNodeInterHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}

		try {
			BnNodeInterHookLocal nf = pHome.findByName(nd.getId(), hookName);
			throw new HeroException("Interactive node hook " + hookName + " already exist in node " + nodeName);
		} catch (javax.ejb.FinderException nn) {
			try {
				BnNodeInterHookLocal h = pHome.create(hookName, eventName, hookType, value);
				h.setBnNode(nd);
			} catch (CreateException fe) {
				throw new HeroException(fe.getMessage());
			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			}
		}
	}

	/**
	 * add interhook to the project
	 *
	 * @param hookName the name of the hook
	 * @param eventName the name of the event
	 * @param hookType the type of the hook
	 * @param script the value of the script 
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void addInterHook(String hookName, String eventName, int hookType, String value) throws HeroException {
		log.debug("addNodeHook: hookName=" + hookName + " eventName=" + eventName + " value=" + value);
		BnProjectInterHookLocalHome pHome;
		BnProjectInterHookLocal ppLocal;

		try {
			pHome = BnProjectInterHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}

		try {
			BnProjectInterHookLocal nf = pHome.findByName(mProject.getId(), hookName);
			throw new HeroException("Interactive hook " + hookName + " already exist in project " + mProject.getName());
		} catch (javax.ejb.FinderException nn) {
			try {
				BnProjectInterHookLocal h = pHome.create(hookName, eventName, hookType, value);
				h.setBnProject(mProject);
			} catch (CreateException fe) {
				throw new HeroException(fe.getMessage());
			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			}
		}
	}

	// -------------------------------------------------------------------------
	// Delete business methods
	// -------------------------------------------------------------------------

	/**
	 * delete a node from the BnProject
	 *
	 * @param name the name ot the edge
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void deleteNode(String name) throws HeroException {
		// check deletion conditions
		log.debug("deleteNode: name=" + name);

		BnNodeLocal nd = mProject.getBnNode(name);
		if (nd.isExecuting()) {
			throw new NodeExecutingException("Cannot delete node. It is executing");
		}
		if (nd.isTerminated()) {
			throw new NodeTerminatedException("Cannot delete node. It is terminated");
		}
		if (nd.isCancelled()) {
			throw new NodeCancelledException("Cannot delete node. It is cancelled");
		}

		Collection edgesIn = nd.getInBnEdges();
		Object[] edai = edgesIn.toArray();
		for (int i = 0; i < edai.length; i++) {
			edgeDeletion((BnEdgeLocal) edai[i]);
		}
		Collection edgesOut = nd.getOutBnEdges();
		Object[] edao = edgesOut.toArray();
		for (int i = 0; i < edao.length; i++) {
			edgeDeletion((BnEdgeLocal) edao[i]);
		}
		try {
			int state = nd.getState();
			int type = nd.getType();
			nd.remove();
			jms.sendNodeEvent(DELETENODE, this.getDetails().getName(), name, type, state, mContext.getCallerPrincipal().getName());
			removeOldNotification(name + ":" + this.getDetails().getName());
		} catch (RemoveException re) {
			throw new HeroException("Cannot remove node " + re.getMessage());
		}
		log.debug("deleteNode -end: name=" + name);
	}

	/**
	 * delete an BnEdge to the BnProject
	 *
	 * @param name the name ot the edge
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void deleteEdge(String name) throws HeroException {
		// check deletion conditions
		log.debug("deleteEdge: name=" + name);
		BnEdgeLocalHome eh;
		BnEdgeLocal ed;
		try {
			eh = BnEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			ed = eh.findByName(name, mProject.getId());
		} catch (FinderException fe) {
			throw new HeroException("edge does not exists " + name);
		}
		String nodeIn = ed.getInBnNode().getName();
		String nodeOut = ed.getOutBnNode().getName();
		edgeDeletion(ed);
		jms.sendEdgeEvent(DELETEEDGE, this.getDetails().getName(), name, nodeIn, nodeOut, mContext.getCallerPrincipal().getName());
		log.debug("deleteEdge -end: name=" + name);
	}

	// Actually deletes an edge and restore the state of the outgoing node
	// Not a business method
	public void edgeDeletion(BnEdgeLocal ed) throws HeroException {
		BnNodeLocal nd = ed.getOutBnNode();
		if (nd.isExecuting()) {
			throw new DeleteEdgeException("Cannot delete edge. Out node executing");
		}
		if (nd.isTerminated()) {
			throw new DeleteEdgeException("Cannot delete edge. Out node terminated");
		}
		if (nd.isCancelled()) {
			throw new DeleteEdgeException("Cannot delete edge. Out node cancelled");
		}
		try {
			ed.remove();
			int newOperation = nd.getActivation().state(nd);
			int newState = nd.getTransition().computeState(nd, newOperation);
			if (newState == hero.interfaces.Constants.Nd.BAD_TRANSITION) {
				throw new HeroException("AddEdge generated a bad BnNode transition");
			}
			this.setNodeState(nd, newState);
		} catch (RemoveException e) {
			throw new HeroException("deleteNode failed " + e.getMessage());
		}
	}

	/**
	 * detete a hook
	 *
	 * @param hookName the name of the hook
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void deleteHook(String hookName) throws HeroException {
		BnProjectHookLocalHome pHome;
		BnProjectHookLocal ppLocal;

		try {
			pHome = BnProjectHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}

		try {
			BnProjectHookLocal ph = pHome.findByName(mProject.getId(), hookName);
			ph.remove();
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		} catch (RemoveException rm) {
			throw new HeroException(rm.getMessage());
		}
	}

	/**
	 * remove a node hook
	 *
	 * @param nodeName the name of the node
	 * @param hookName the name 
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void deleteNodeHook(String nodeName, String hookName) throws HeroException {
		BnNodeHookLocalHome pHome;
		BnNodeHookLocal ppLocal;
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		try {
			pHome = BnNodeHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			BnNodeHookLocal nh = pHome.findByName(nd.getId(), hookName);
			nh.remove();
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		} catch (RemoveException rm) {
			throw new HeroException(rm.getMessage());
		}
	}

	/**
	 * Delete a property of the project
	 *
	 * @param key the key of the property
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void deleteProperty(String key) throws HeroException {
		log.debug("deleteProperty key=" + key);

		BnProjectPropertyLocalHome pHome;
		BnProjectPropertyLocal ppLocal;

		try {
			pHome = BnProjectPropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(mProject.getId(), key);
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}

		if (!c.isEmpty()) {
			try {
				ppLocal = (BnProjectPropertyLocal) (c.toArray())[0];
				ppLocal.remove();
			} catch (RemoveException rm) {
				throw new HeroException(rm.getMessage());
			}

		} else {
			throw new HeroException("Property not found :" + key);
		}
	}

	/**
	 * Delete a property of a node
	 *
	 * @param nodeName the name of the node
	 * @param key the key of the property
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void deleteNodeProperty(String nodeName, String key) throws HeroException {
		log.debug("deleteNodeProperty node=" + nodeName + " key=" + key);

		BnNodePropertyLocalHome pHome;
		BnNodePropertyLocal propertyLocal = null;

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		try {
			pHome = BnNodePropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(nd.getId(), key);
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}

		if (!c.isEmpty()) {
			try {
				propertyLocal = (BnNodePropertyLocal) (c.toArray())[0];
				propertyLocal.remove();
			} catch (RemoveException rm) {
				throw new HeroException(rm.getMessage());
			}

		} else {
			throw new HeroException("Property not found :" + key);
		}
	}

	// -------------------------------------------------------------------------
	// Set business methods
	// -------------------------------------------------------------------------

	/**
	 * Add a role to the project
	 *
	 * @param userName the name of the user
	 * @param roleName the name of the role
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setUserRole(String userName, String roleName) throws HeroException {
		log.debug("setUserRole: user=" + userName + " role=" + roleName);
		BnRoleLocalHome roleh;
		BnUserLocalHome userh;
		try {
			roleh = BnRoleUtil.getLocalHome();
			userh = BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			/*NamingManagementLocal namingManagementLocal = NamingManagementUtil.getLocalHome().create();			
			BnUserLocal user = userh.findByName(namingManagementLocal.getUsersRootName() + namingManagementLocal.getSeparator()+userName);*/
			BnUserLocal user = userh.findByName(userName);		
			BnRoleLocal role = roleh.findByName(roleName, mProject.getId());
			user.getBnRoles().add(role);
			jms.sendUserRoleEvent(SETUSERROLE, this.getDetails().getName(), userName, roleName);
		} catch (Exception fe) {
			throw new HeroException(fe.getMessage());
		}
	}

	/**
	 * Propagate BnNode properties 
	 * @param nodeName the name of the BnNode
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void propagateNodeProperties(String nodeName) throws HeroException {
		log.debug("propagateNodeProperties: name=" + nodeName);

		BnNodePropertyLocalHome pHome;
		BnNodePropertyLocal propertyLocal = null;

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		Collection c = nd.getBnProperties();
		Collection c2 = nd.getOutBnEdges();
		if ((!c.isEmpty()) && (!c2.isEmpty())) {
			BnNodePropertyLocal p;
			BnEdgeLocal ed;
			for (Iterator i = c.iterator(); i.hasNext();) {
				p = (BnNodePropertyLocal) i.next();
				if (p.getPropagate()) {
					for (Iterator i2 = c2.iterator(); i2.hasNext();) {
						ed = (BnEdgeLocal) i2.next();
						this.setNodeProperty((ed.getOutBnNode()).getName(), p.getTheKey(), p.getTheValue());
					}
				}
			}
		}
	}

	/**
	 * Sets the role of an activity
	 *
	 * @param name the name of the activity
	 * @param role the name of the role
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setNodeRole(String name, String role) throws HeroException {
		log.debug("setBnNodeRole: name=" + name + " role=" + role);
		BnRoleLocalHome rh;
		BnNodeLocalHome nh;
		BnNodeLocal nl;
		BnRoleLocal rl;
		try {
			rh = BnRoleUtil.getLocalHome();
			nh = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			nl = nh.findByName(name, mProject.getId());
		} catch (FinderException fe) {
			throw new HeroException("BnNode does not exists " + name);
		}
		try {
			rl = rh.findByName(role, mProject.getId());
		} catch (FinderException fe) {
			throw new HeroException("BnRole does not exists " + role);
		}
		nl.setBnRole(rl);
		jms.sendProjectEvent(CHANGEROLE, this.getDetails().getName());
	}

	/**
	 * Set the node State
	 *
	 * @param name of the node
	 * @param new state of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setNodeState(BnNodeLocal node, int state) throws HeroException {
		log.debug("setBnNodeState: name=" + node.getName() + " state=" + state);
		node.setState(state);
		if (state == hero.interfaces.Constants.Nd.EXECUTING) {
			node.setStartDate(new java.util.Date());
		}
		if (state == hero.interfaces.Constants.Nd.TERMINATED) {
			node.setEndDate(new java.util.Date());
			this.propagateNodeProperties(node.getName());
		}
		jms.sendNodeEvent(SETNODESTATE, this.getDetails().getName(), node.getName(), node.getType(), state, mContext.getCallerPrincipal().getName());

	}

	/**
	 * Set the edge State
	 *
	 * @param bean of the edge
	 * @param new state of the edge
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setEdgeState(BnEdgeLocal edge, int state) throws HeroException {
		log.debug("setEdgeState: name=" + edge.getName() + " state=" + state);
		edge.setState(state);
		jms.sendEdgeEvent(
			SETEDGESTATE,
			this.getDetails().getName(),
			edge.getName(),
			edge.getInBnNode().getName(),
			edge.getOutBnNode().getName(),
			mContext.getCallerPrincipal().getName());
	}

	/**
	 * Set the BnAgent State
	 *
	 * @param bean of the BnAgent
	 * @param new state of the BnAgent
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setAgentState(BnAgentLocal agent, int state) throws HeroException {
		log.debug("setBnAgentState: name=" + agent.getName() + " state=" + state);
		agent.setState(state);
		//	jms.sendEdgeEvent(SETEDGESTATE,this.getDetails().getName(),edge.getName(),edge.getInBnNode().getName(),edge.getOutBnNode().getName(),mContext.getCallerPrincipal().getName());
	}

	/**
	 * Set the BnAgentEdge State
	 *
	 * @param bean of the BnAgentEdge
	 * @param new state of the BnAgentEdge
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setAgentEdgeState(BnAgentEdgeLocal agentEdge, int state) throws HeroException {
		log.debug("setBnAgentEdgeState: name=" + agentEdge.getName() + " state=" + state);
		agentEdge.setState(state);
		//	jms.sendEdgeEvent(SETEDGESTATE,this.getDetails().getName(),edge.getName(),edge.getInBnNode().getName(),edge.getOutBnNode().getName(),mContext.getCallerPrincipal().getName());
	}

	/**
	 * Set the edge Condition
	 *
	 * @param bean of the edge
	 * @param new state of the edge
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setEdgeCondition(String edge, String condition) throws HeroException {
		log.debug("setEdgeState: name=" + edge + " condition=" + condition);
		BnEdgeLocal e = mProject.getBnEdge(edge);
		e.setCondition(condition);
	}

	/**
	 * Set the node deadline
	 *
	 * @param name of the node
	 * @param Date of the deadline
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setNodeDeadline(String name, long date) throws HeroException {
			log.debug("setBnNodeDeadline: name=" + name + " deadline=" + date);

		
			BnNodeLocal nd = mProject.getBnNode(name);
			nd.setDeadline(new java.sql.Date(date));
			String projectName = this.getDetails().getName();
			jms.sendNodeEvent(SETNODEDEADLINE, projectName, name, nd.getType(), nd.getState(), mContext.getCallerPrincipal().getName());

			try {
				String type = name + ":" + projectName;
				removeOldNotification(type);

/*				ObjectName timer = new ObjectName("jboss:service=timer");
				ObjectName receiver = new ObjectName("jboss:service=TimerReceiver");*/
				ObjectName timer    = new ObjectName("jonas:type=service,name=Timer");
				ObjectName receiver    = new ObjectName("jonas:type=service,name=TimerReceiver");
				List list = MBeanServerFactory.findMBeanServer(null);
				MBeanServer server = (MBeanServer) list.iterator().next();
				Date lNext = new Date(date);
				NotificationFilter filter = new TimerFilter();
				server.addNotificationListener(timer, receiver, filter, null);
				Integer id = (Integer) server.invoke(timer, "addNotification", new Object[] { type, //type
					name, //message
					"bonita", //user object
					lNext, //delay time
				}, new String[] { "".getClass().getName(), "".getClass().getName(), "java.lang.Object", Date.class.getName()});
			} catch (Exception e) {e.printStackTrace();
				throw new HeroException(e.getMessage());
			}

		}
	
	private void removeOldNotification(String name) throws HeroException {
		try {
			ObjectName timer = new ObjectName("bonita.jmx:service=timer");
			List list = MBeanServerFactory.findMBeanServer(null);
			MBeanServer server = (MBeanServer) list.iterator().next();

			server.invoke(timer, "removeNotifications", new Object[] { name }, new String[] { "".getClass().getName()});
		} catch (javax.management.InstanceNotFoundException ex) {
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}

	/**
	 * Set the node description
	 *
	 * @param name of the node
	 * @param description of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setNodeDescription(String name, String description) throws HeroException {
		log.debug("setBnNodeDescription: name=" + name + " description=" + description);
		BnNodeLocal nd = mProject.getBnNode(name);
		nd.setDescription(description);
	}
	

	/**
	 * Set the node hook value. Only for interactives hooks
	 *
	 * @param name of the node
	 * @param description of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setNodeInterHookValue(String node, String hook, String value) throws HeroException {
		log.debug("setBnNodeInterHookValue: node=" + node + " hook=" + hook + " value=" + value);
		BnNodeLocal nd = mProject.getBnNode(node);
		Collection hooks = nd.getBnInterHooks();
		for (Iterator ih = hooks.iterator(); ih.hasNext();) {
			BnNodeInterHookLocal hk = (BnNodeInterHookLocal) ih.next();
			if (hk.getName().equals(hook)) {
				hk.setScript(value);
			}
		}
	}
	

	/**
	 * Set the hook value. Only for interactives hooks
	 *
	 * @param description of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setInterHookValue(String hook, String value) throws HeroException {
		log.debug("setBnNodeInterHookValue:  hook=" + hook + " value=" + value);
		Collection hooks = mProject.getBnInterHooks();
		for (Iterator ih = hooks.iterator(); ih.hasNext();) {
			BnProjectInterHookLocal hk = (BnProjectInterHookLocal) ih.next();
			if (hk.getName().equals(hook)) {
				hk.setScript(value);
			}
		}
	}

	public void unsetUserRole(String userName, String roleName) {
		log.debug("unsetUserRole: userName=" + userName + " roleName=" + roleName);

		//	jms.sendUserRoleEvent(UNSETUSERROLE,this.getDetails().getName(),userName,roleName);
	}

	/**
	 * Set the node in traditional mode
	 *
	 * @param name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setNodeTraditional(String name) throws HeroException {
		log.debug("setBnNodeTraditional: name=" + name);
		BnNodeLocal nd = mProject.getBnNode(name);
		nd.setAnticipable(false);
		nd.setTransition(NodeState.make(nd.getType(), false));

	}

	/**
	 * Set the node in anticipable mode
	 *
	 * @param name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setNodeAnticipable(String name) throws HeroException {
		log.debug("setBnNodeAnticipable: name=" + name);
		BnNodeLocal nd = mProject.getBnNode(name);
		nd.setAnticipable(true);
		nd.setTransition(NodeState.make(nd.getType(), true));
	}

	/**
	 * Set the node in automatic  mode
	 *
	 * @param name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setNodeAutomatic(String name) throws HeroException {
		log.debug("setBnNodeAutomatic: name=" + name);
		BnNodeLocal nd = mProject.getBnNode(name);
		//nd.setAutomatic(true);
		nd.setTransition(NodeState.make(5, true));
		//nd.setTransition(NodeState.make(nd.getType(),true));
		//	this.addNodeHok(name,"hero.hook.AutomaticHook",hero.interfaces.Constants.Nd.CREATE,Hook.JAVA);
	}

	/**
	 * Set the name of the node executor
	 *
	 * @param name the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setNodeExecutor(String name) throws HeroException {
		log.debug("setBnNodeExecutor: name=" + name);
		BnUserLocalHome uHome;
		BnUserLocal user;
		try {
			uHome = BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		BnNodeLocal nd = mProject.getBnNode(name);
		String executor = mContext.getCallerPrincipal().getName();
		try {
			user = uHome.findByName(executor);
		} catch (FinderException fe) {
			throw new HeroException("BnUser " + executor + " does not exists");
		}
		nd.setExecutor(user);
		jms.sendNodeEvent(SETNODEEXECUTOR, this.getDetails().getName(), name, nd.getType(), nd.getState(), mContext.getCallerPrincipal().getName());
	}

	/**
	 * Set the node type
	 *
	 * @param name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setNodeType(String name, int type) throws HeroException {
		log.debug("setBnNodeType: name=" + name + " type" + type);
		BnNodeLocal nd = mProject.getBnNode(name);
		nd.setType(type);
		nd.setTransition(NodeState.make(nd.getType(), true));
		nd.setActivation(EdgeState.make(nd.getType()));
	}

	/**
	 * Set a property of a node
	 *
	 * @param nodeName the name of the node
	 * @param key the key of the property
	 * @param value the name of the property
	 * @param propagate if it is a propagation property
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setNodeProperty(String nodeName, String key, String value, boolean propagate) throws HeroException {

		BnNodePropertyLocalHome pHome;
		BnNodePropertyLocal propertyLocal;

		log.debug("setBnNodeProperty node=" + nodeName + " key=" + key + " value=" + value);

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		try {
			pHome = BnNodePropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(nd.getId(), key);
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
		if (!c.isEmpty()) {
			propertyLocal = (BnNodePropertyLocal) (c.toArray())[0];
			propertyLocal.setTheValue(value);
			propertyLocal.setPropagate(propagate);
		} else {
			try {
				propertyLocal = pHome.create(key, value);
				propertyLocal.setBnNode(nd);
				propertyLocal.setPropagate(propagate);
			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			} catch (javax.ejb.CreateException ce) {
				throw new HeroException(ce.getMessage());
			}
		}
	}

	/**
	 * Set a property of a node
	 *
	 * @param nodeName the name of the node
	 * @param key the key of the property
	 * @param value the name of the property
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/

	public void setNodeProperty(String nodeName, String key, String value) throws HeroException {
		BnNodePropertyLocalHome pHome;
		BnNodePropertyLocal propertyLocal = null;

		log.debug("setBnNodeProperty node=" + nodeName + " key=" + key + " value=" + value);
		BnNodeLocal nd = mProject.getBnNode(nodeName);

		try {
			pHome = BnNodePropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(nd.getId(), key);
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}

		if (!c.isEmpty()) {

			propertyLocal = (BnNodePropertyLocal) (c.toArray())[0];
			propertyLocal.setTheValue(value);

		} else {
			try {
				propertyLocal = pHome.create(key, value);
				propertyLocal.setBnNode(nd);

			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			} catch (javax.ejb.CreateException ce) {
				throw new HeroException(ce.getMessage());
			}
		}
	}

	/**
	 * Set a property of the project
	 *
	 * @param key the key of the property
	 * @param value the name of the property
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setProperty(String key, String value) throws HeroException {
		log.debug("setProperty key=" + key + " value=" + value);

		BnProjectPropertyLocalHome pHome;
		BnProjectPropertyLocal ppLocal = null;

		try {
			pHome = BnProjectPropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(mProject.getId(), key);
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}

		if (!c.isEmpty()) {

			ppLocal = (BnProjectPropertyLocal) (c.toArray())[0];
			ppLocal.setTheValue(value);

		} else {
			try {
				ppLocal = pHome.create(key, value);
				ppLocal.setBnProject(mProject);

			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			} catch (javax.ejb.CreateException ce) {
				throw new HeroException(ce.getMessage());
			}
		}
	}

	// -------------------------------------------------------------------------
	// Get business methods
	// -------------------------------------------------------------------------

	/**
	 * Get the node description
	 *
	 * @param name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getNodeDescription(String name) throws HeroException {
		log.debug("setBnNodeDescription: name=" + name);
		BnNodeLocal nd = mProject.getBnNode(name);
		return(nd.getDescription());
	}


	/**
	 * Get the node hook value. Only for interactives hooks
	 *
	 * @param name of the node
	 * @param hook of the node
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public String getNodeInterHookValue(String node, String hook) throws HeroException {
		String value =null;
		log.debug("getBnNodeInterHookValue: node=" + node + " hook=" + hook);
		BnNodeLocal nd = mProject.getBnNode(node);
		Collection hooks = nd.getBnInterHooks();
		for (Iterator ih = hooks.iterator(); ih.hasNext();) {
			BnNodeInterHookLocal hk = (BnNodeInterHookLocal) ih.next();
			if (hk.getName().equals(hook)) {
				value = hk.getScript();
			}
		}
		return value;
	}

	/**
	 * Get the BnProjectValue (historique)
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public BnProjectValue getDetails() throws HeroException {
		log.debug("getDetails:");
		return mProject.getBnProjectValue();
	}

	/**
	 * Get ProjectName
	 * @return String 
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public String getName() throws HeroException {
		log.debug("getName:");
		return mProject.getName();
	}

	/**
	 * Get creator
	 * @return String 
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public String getCreator() throws HeroException {
		log.debug("getCreator:");
		return mProject.getCreator();
	}

	/**
	 * Get all roles of the project
	 * @return Collection 
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getRoles() throws HeroException {
		log.debug("getBnRoles:");
		Collection roles = mProject.getBnRoles();
		ArrayList result = new ArrayList();
		for (Iterator i = roles.iterator(); i.hasNext();) {
			result.add((BnRoleLocal) i.next());
		}
		return result;
	}

	/**
	 * Get all roles names of the project
	 * @return Collection 
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getRolesNames() throws HeroException {
		log.debug("getBnRolesNames:");
		Collection roles = mProject.getBnRoles();
		ArrayList result = new ArrayList();
		for (Iterator i = roles.iterator(); i.hasNext();) {
			result.add(((BnRoleLocal) i.next()).getName());
		}
		return result;
	}

	/**
	 * Get all user roles
	 * @return Collection 
	 *
	 * @param userName The name of the user
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getUserRoles(String userName) throws HeroException {
		log.debug("getUserRole:");
		BnUserLocalHome uHome;
		BnUserLocal user;
		try {
			uHome = (BnUserLocalHome) hero.interfaces.BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			user = uHome.findByName(userName);
		} catch (FinderException f) {
			throw new HeroException("FinderException");
		}
		Collection roles = user.getBnRoles();
		ArrayList result = new ArrayList();
		for (Iterator i = roles.iterator(); i.hasNext();) {
			result.add((BnRoleLocal) i.next());
		}
		return result;
	}

	/**
	 * Get user roles in project
	 * @return Collection 
	 *
	 * @param userName The name of the user
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getUserRolesInProject(String userName) throws HeroException {
		log.debug("getUserRolesInProject:");
		BnUserLocalHome uHome;
		BnUserLocal user;
		ArrayList result = new ArrayList();
		try {
			uHome = (BnUserLocalHome) hero.interfaces.BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			user = uHome.findByName(userName);
		} catch (FinderException f) {
			throw new HeroException("FinderException");
		}
		Collection userRoles = user.getBnRoles();
		Collection projectRoles = mProject.getBnRoles();
		for (Iterator i = userRoles.iterator(); i.hasNext();) {
			BnRoleLocal ur = (BnRoleLocal) i.next();
			for (Iterator j = projectRoles.iterator(); j.hasNext();) {
				BnRoleLocal up = (BnRoleLocal) j.next();
				if (ur.getId() == up.getId())
					result.add(ur.getBnRoleValue());
			}
		}
		return result;
	}

	/**
	 * Get user roles in project names
	 * @return Collection 
	 *
	 * @param userName The name of the user
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getUserRolesInProjectNames(String userName) throws HeroException {
		log.debug("getUserRolesInProjectNames:");
		BnUserLocalHome uHome;
		BnUserLocal user;
		ArrayList result = new ArrayList();
		try {
			uHome = (BnUserLocalHome) hero.interfaces.BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			user = uHome.findByName(userName);
		} catch (FinderException f) {
			throw new HeroException("FinderException");
		}
							
		Collection userRoles = user.getBnRoles();
		Collection projectRoles = mProject.getBnRoles();
		for (Iterator i = userRoles.iterator(); i.hasNext();) {
			BnRoleLocal ur = (BnRoleLocal) i.next();
			for (Iterator j = projectRoles.iterator(); j.hasNext();) {
				BnRoleLocal up = (BnRoleLocal) j.next();
				if (ur.getId() == up.getId())
					result.add(ur.getBnRoleValue().getName());
			}
		}		
		return result;
	}

	/**
	 * Get node role
	 * @return String
	 *
	 * @param nodeName The name of the node
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public BnRoleLocal getNodeRole(String nodeName) throws HeroException {
		log.debug("getBnNodeRole: nodeName=" + nodeName);
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		return ((BnRoleLocal) nd.getBnRole());
	}

	/**
	 * Get node role name
	 * @return String
	 *
	 * @param nodeName The name of the node
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public String getNodeRoleName(String nodeName) throws HeroException {
		log.debug("getBnNodeRoleName: nodeName=" + nodeName);
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		return (((BnRoleLocal) nd.getBnRole()).getName());
	}

	/**
	 * Get node deadline
	 * @return String 
	 *
	 * @param nodeName The name of the node
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public String getNodeDeadline(String nodeName) throws HeroException {
		log.debug("getBnNodeDeadline: nodeName=" + nodeName);
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		if (nd.getDeadline() != null)
			return (nd.getDeadline().toString());
		else
			return (null);
	}

	/**
	 * Get the node Value
	 *
	 * @param name the name of the BnNode
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public BnNodeValue getNodeValue(String name) throws HeroException {
		log.debug("getBnNode: name=" + name);
		BnNodeLocalHome nHome;
		BnNodeLocal mNode;
		BnNodeValue nv;
		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			mNode = nHome.findByName(name, mProject.getId());
		} catch (FinderException fe) {
			throw new HeroException("In BnNode does not exist");
		}
		nv = mNode.getBnNodeValue();

		return nv;
	}

	/**
	 * Get the BnAgent Value
	 *
	 * @param name the name of the BnAgent
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public BnAgentValue getAgentValue(String name) throws HeroException {
		log.debug("getBnAgent: name=" + name);
		BnAgentLocalHome nHome;
		BnAgentLocal mAgent;
		BnAgentValue av;
		try {
			nHome = hero.interfaces.BnAgentUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			mAgent = nHome.findByName(name, mProject.getId());
		} catch (FinderException fe) {
			throw new HeroException("In BnAgent does not exist");
		}
		av = mAgent.getBnAgentValue();

		return av;
	}

	/**
	 * Get the BnAgentEdge Value
	 *
	 * @param name the name of the BnAgentEdge
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public BnAgentEdgeValue getAgentEdgeValue(String name) throws HeroException {
		log.debug("getBnAgentEdge: name=" + name);
		BnAgentEdgeLocalHome nHome;
		BnAgentEdgeLocal mAgentEdge;
		BnAgentEdgeValue av;
		try {
			nHome = hero.interfaces.BnAgentEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			mAgentEdge = nHome.findByName(name, mProject.getId());
		} catch (FinderException fe) {
			throw new HeroException("In BnAgentEdge does not exist");
		}
		av = mAgentEdge.getBnAgentEdgeValue();

		return av;
	}

	/**
	 * Get the node Light Value
	 *
	 * @param name the name of the BnNode
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public BnNodeLightValue getNodeLightValue(String name) throws HeroException {
		log.debug("getBnNode: name=" + name);
		BnNodeLocalHome nHome;
		BnNodeLocal mNode;
		BnNodeLightValue nv;
		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			mNode = nHome.findByName(name, mProject.getId());
		} catch (FinderException fe) {
			throw new HeroException("In BnNode does not exist");
		}
		nv = mNode.getBnNodeLightValue();

		return nv;
	}

	/**
	 * Get the StrutsNode 
	 *
	 * @param nodeName the name of the BnNode
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public StrutsNodeValue getStrutsNode(String nodeName) throws HeroException {
		log.debug("getStrutsNode: nodeName=" + nodeName);

		BnNodeLocalHome nHome;
		BnNodeLocal n;
		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			n = nHome.findByName(nodeName, mProject.getId());

			StrutsNodeValue snv = new StrutsNodeValue();
			snv.setProjectName(mProject.getName());
			snv.setName(n.getName());
			snv.setCreator(n.getCreator().getName());
			snv.setRole(n.getBnRole().getName());
			snv.setState(hero.interfaces.Constants.Nd.nodeStateName[n.getState()]);
			snv.setType(hero.interfaces.Constants.Nd.nodeTypeName[n.getType()]);
			if (n.getDescription() != null)
				snv.setDescription(n.getDescription());
			if (n.getExecutor() != null)
				snv.setExecutor(n.getExecutor().getName());
			snv.setAnticipable(n.getAnticipable());
			if (n.getDeadline() != null)
				snv.setDeadline(n.getDeadline().toString());
	        
			return (snv);
		} catch (FinderException fe) {
			throw new HeroException("In BnNode does not exist");
		}
	}

	/**
	 * Get the BnEdge Value
	 *
	 * @param name the name of the BnEdge
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public BnEdgeValue getEdgeValue(String name) throws HeroException {
		log.debug("getBnEdge: name=" + name);
		BnEdgeLocalHome eHome;
		BnEdgeLocal mEdge;
		BnEdgeValue ev;
		try {
			eHome = hero.interfaces.BnEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			mEdge = eHome.findByName(name, mProject.getId());
		} catch (FinderException fe) {
			throw new HeroException("BnEdge does not exist");
		}
		ev = mEdge.getBnEdgeValue();
		return ev;
	}

	/**
	 * Set the node executor
	 *
	 * @param name of the node
	 * @return The user executor
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public String getNodeExecutor(String name) throws HeroException {
		log.debug("setBnNodeDescription: name=" + name);
		BnNodeLocal nd = mProject.getBnNode(name);
		BnUserLocal ul = nd.getExecutor();
		return (ul.getName());
	}

	/**
	 * Get the state of the node
	 *
	 * @param name of the node
	 * @return The state of the node
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public int getNodeState(String name) throws HeroException {
		log.debug("setBnNodeState: name=" + name);
		BnNodeLocal nd = mProject.getBnNode(name);
		return (nd.getState());
	}

	/**
	 * Get the type of the node
	 *
	 * @param name of the node
	 * @return The type of the node
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public int getNodeType(String name) throws HeroException {
		log.debug("setBnNodeType: name=" + name);
		BnNodeLocal nd = mProject.getBnNode(name);
		return (nd.getType());
	}

	/**
	 * Get all users of the project
	 * @return Collection 
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getUsers() throws HeroException {
		log.debug("getUsers:");
		Collection users = mProject.getBnUsers();
		ArrayList result = new ArrayList();
		for (Iterator i = users.iterator(); i.hasNext();) {
			result.add(((BnUserLocal) i.next()).getName());
		}
		return result;
	}

	/**
	  * Return all registered users
	  * @return A string collection of all user names
	  *
	  * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	  *
	  **/
	public Collection getAllUsers() throws HeroException {
		log.debug("getAllUsers:");
		BnUserLocalHome uHome;
		Collection users;
		try {
			uHome = (BnUserLocalHome) hero.interfaces.BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			users = uHome.findAll();
		} catch (FinderException f) {
			throw new HeroException("FinderException in findByAll users");
		}
		ArrayList result = new ArrayList();
		for (Iterator i = users.iterator(); i.hasNext();) {
			result.add(((BnUserLocal) i.next()).getName());
		}
		return result;
	}

	/**
	 * Get all node names of the project
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getNodesNames() throws HeroException {
		log.debug("getBnNodesNames: ");

		//	Vector names = new Vector();
		ArrayList names = new ArrayList();
		Collection nodes = mProject.getBnNodes();
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			BnNodeLocal n = (BnNodeLocal) i.next();
			names.add(n.getName());
		}
		return ((Collection) names);
	}

	/**
	 * Get all edges of the project
	 * @return Collection 
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getEdgesNames() throws HeroException {
		log.debug("getBnEdges:");
		//	Vector names = new Vector();
		ArrayList names = new ArrayList();
		Collection edges = mProject.getBnEdges();
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnEdgeLocal n = (BnEdgeLocal) i.next();
			names.add(n.getName());
		}
		return ((Collection) names);
	}

	/**
	 * Get edge in node
	 * @return String 
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public String getEdgeInNode(String edgeName) throws HeroException {
		log.debug("getBnEdgeInNode: edgeName" + edgeName);
		BnEdgeLocal e = mProject.getBnEdge(edgeName);
		return (((BnNodeLocal) e.getInBnNode()).getName());
	}

	/**
	 * Get edge out node
	 * @return String 
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public String getEdgeOutNode(String edgeName) throws HeroException {
		log.debug("getBnEdgeOutNode: edgeName" + edgeName);
		BnEdgeLocal e = mProject.getBnEdge(edgeName);
		return (((BnNodeLocal) e.getOutBnNode()).getName());
	}

	/**
	 * get all in edges of the node
	 * @return Collection 
	 *
	 * @param name the name ot the node
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getNodeInEdges(String name) throws HeroException {
		log.debug("getBnNodeInEdges: name=" + name);

		Collection in = new ArrayList();
		BnNodeLocal nd = mProject.getBnNode(name);

		Collection edgesIn = nd.getInBnEdges();
		Iterator i = edgesIn.iterator();
		while (i.hasNext())
			in.add(((BnEdgeLocal) i.next()).getName());

		return (in);
	}

	/**
	 * get all out edges of the node
	 * @return Collection 
	 *
	 * @param name the name ot the node
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getNodeOutEdges(String name) throws HeroException {
		log.debug("getBnNodeOutEdges: name=" + name);

		Collection out = new ArrayList();
		BnNodeLocal nd = mProject.getBnNode(name);

		Collection edgesOut = nd.getOutBnEdges();
		Iterator i = edgesOut.iterator();
		while (i.hasNext())
			out.add(((BnEdgeLocal) i.next()).getName());

		return (out);
	}

	/**
	 * Get BnNode properties of the project
	 *
	 * @param nodeName the name of the node
	 * @return A BnNodePropertyValue collection of all node properties
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getNodeProperties(String nodeName) throws HeroException {
		Collection prp = new ArrayList();
		BnNodePropertyLocal pl;

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		Collection pls = nd.getBnProperties();

		for (Iterator i = pls.iterator(); i.hasNext();) {
			pl = (BnNodePropertyLocal) i.next();
			prp.add(pl.getBnNodePropertyValue());
		}
		return (prp);
	}

	/**
	 * Get BnNode hooks of the project
	 *
	 * @param nodeName the name of the node
	 * @return A BnNodeHookValue collection of all node hooks
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getNodeHooks(String nodeName) throws HeroException {
		Collection hk = new ArrayList();
		BnNodeHookLocal pl;

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		Collection pls = nd.getBnHooks();

		for (Iterator i = pls.iterator(); i.hasNext();) {
			pl = (BnNodeHookLocal) i.next();
			hk.add(pl.getBnNodeHookValue());
		}
		return (hk);
	}

	/**
	 * Get Interactive BnNode hooks of the project
	 *
	 * @param nodeName the name of the node
	 * @return A BnNodeHookValue collection of all node hooks
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getNodeInterHooks(String nodeName) throws HeroException {
		Collection hk = new ArrayList();
		BnNodeInterHookLocal pl;

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		Collection pls = nd.getBnInterHooks();

		for (Iterator i = pls.iterator(); i.hasNext();) {
			pl = (BnNodeInterHookLocal) i.next();
			hk.add(pl.getBnNodeInterHookValue());
		}
		return (hk);
	}

	/**
	 * Get properties of the project
	 * @return A BnProjectPropertyValue collection of all project properties
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getProperties() throws HeroException {
		Collection prp = new ArrayList();
		BnProjectPropertyLocal ppl;

		Collection pls = mProject.getBnProperties();
		for (Iterator i = pls.iterator(); i.hasNext();) {
			ppl = (BnProjectPropertyLocal) i.next();
			prp.add(ppl.getBnProjectPropertyValue());
		}
		return (prp);
	}

	/**
	 * Get hooks of the project
	 * @return A ProjectHooksValue collection of all project hook
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getHooks() throws HeroException {
		Collection hooks = new ArrayList();
		BnProjectHookLocal phl;

		Collection hks = mProject.getBnHooks();
		for (Iterator i = hks.iterator(); i.hasNext();) {
			phl = (BnProjectHookLocal) i.next();
			hooks.add(phl.getBnProjectHookValue());
		}
		return (hooks);
	}

	/**
	 * Execute a hook
	 *
	 * @param nodeName the name of the node
	 * @param event the event 
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public void hookEvent(String nodeName, String event) throws HeroException {
		log.debug("hookEvent: node=" + nodeName + " event=" + event);
		Collection hooks;
		BnNodeLocal node;

		node = mProject.getBnNode(nodeName);
		hooks = node.getBnHooks();
		for (Iterator i = hooks.iterator(); i.hasNext();) {
			BnNodeHookLocal hk = (BnNodeHookLocal) i.next();
			if (hk.getEvent().equalsIgnoreCase(event)) {
				hero.hook.Hook h = hero.hook.Hook.make(hk.getName(), event, hk.getType());
				h.execute(this, event, node);
			}
		}
	}

	// -------------------------------------------------------------------------
	// Struts business methods
	// -------------------------------------------------------------------------

	/**
	 * Get the StrutsNodes
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getStrutsNodes() throws HeroException {
		log.debug("getStrutsNodes: ");

		//ctor strutsNodes = new Vector();
		ArrayList strutsNodes = new ArrayList();
		Collection nodes = mProject.getBnNodes();
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			BnNodeLocal n = (BnNodeLocal) i.next();
			StrutsNodeValue snv = new StrutsNodeValue();
			snv.setName(n.getName());
			snv.setCreator(n.getCreator().getName());
			snv.setRole(n.getBnRole().getName());
			snv.setState(hero.interfaces.Constants.Nd.nodeStateName[n.getState()]);
			snv.setType(hero.interfaces.Constants.Nd.nodeTypeName[n.getType()]);
			snv.setDescription(n.getDescription());
			if (n.getExecutor() != null)
				snv.setExecutor(n.getExecutor().getName());
			snv.setAnticipable(n.getAnticipable());
			if (n.getDeadline() != null)
				snv.setDeadline(n.getDeadline().toString());
			strutsNodes.add(snv);
		}
		return ((Collection) strutsNodes);
	}

	/**
	 * Get the Edges of the BnNode
	 *
	 * @param nodeName the name of the BnNode
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getStrutsNodeEdges(String nodeName) throws HeroException {
		log.debug("getStrutsNodeEdges: nodeName=" + nodeName);

		BnNodeLocalHome nHome;
		BnNodeLocal mNode;
		BnNodeValue nv;

		//	Vector edges = new Vector();
		ArrayList edges = new ArrayList();

		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			mNode = nHome.findByName(nodeName, mProject.getId());
		} catch (FinderException fe) {
			throw new HeroException("In BnNode does not exist");
		}
		Collection in = mNode.getInBnEdges();
		Collection out = mNode.getOutBnEdges();

		Iterator iin = in.iterator();
		while (iin.hasNext()) {
			StrutsEdgeValue sev = new StrutsEdgeValue();
			hero.interfaces.BnEdgeLocal actualEdge =
				(hero.interfaces.BnEdgeLocal) iin.next();
			sev.setState(hero.interfaces.Constants.Ed.edgeStateName[actualEdge.getState()]);
			sev.setInNode(((BnNodeLocal) actualEdge.getInBnNode()).getName());
			sev.setOutNode(((BnNodeLocal) actualEdge.getOutBnNode()).getName());
			edges.add(sev);
		}

		Iterator iout = out.iterator();
		while (iout.hasNext()) {
			StrutsEdgeValue sev = new StrutsEdgeValue();
			BnEdgeLocal actualEdge = (BnEdgeLocal) iout.next();
			sev.setState(hero.interfaces.Constants.Ed.edgeStateName[actualEdge.getState()]);
			sev.setInNode(((BnNodeLocal) actualEdge.getInBnNode()).getName());
			sev.setOutNode(((BnNodeLocal) actualEdge.getOutBnNode()).getName());
			edges.add(sev);
		}

		return ((Collection) edges);
	}

	/**
	 * Get the StrutsEdges
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getStrutsEdges() throws HeroException {
		log.debug("getStrutsEdges: ");

		//	Vector strutsEdges = new Vector();
		ArrayList strutsEdges = new ArrayList();
		Collection edges = mProject.getBnEdges();
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnEdgeLocal e = (BnEdgeLocal) i.next();
			StrutsEdgeValue sev = new StrutsEdgeValue();
			sev.setName(e.getName());
			sev.setState(hero.interfaces.Constants.Ed.edgeStateName[e.getState()]);
			sev.setInNode(e.getInBnNode().getName());
			sev.setOutNode(e.getOutBnNode().getName());
			strutsEdges.add(sev);
		}
		return ((Collection) strutsEdges);
	}

	/**
	 * Get the project nodes
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/	
	public Object[] getNodes() throws HeroException {

		//ProjectNodes nb = new ProjectNodes();		
		ArrayList ns = new ArrayList();
		Collection nodes = new ArrayList();	
				
		nodes = mProject.getBnNodes();
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			BnNodeLocal n = (BnNodeLocal) i.next();
			StrutsNodeValue snv = new StrutsNodeValue();
			snv.setProjectName(mProject.getName());
			snv.setName(n.getName());
			snv.setCreator(n.getCreator().getName());
			snv.setRole(n.getBnRole().getName());
			snv.setState(Constants.Nd.nodeStateName[n.getState()]);
			snv.setType(Constants.Nd.nodeTypeName[n.getType()]);
			if (n.getDescription() != null)
				snv.setDescription(n.getDescription());
			if (n.getExecutor() != null)
			    snv.setExecutor(n.getExecutor().getName());
			snv.setAnticipable(n.getAnticipable());
			if (n.getDeadline() != null)
				snv.setDeadline(n.getDeadline().toString());
						
			ns.add(snv);
		}
		//nb.setNodes(ns.toArray());
		return (ns.toArray());
	}	

	/**
	 * import all the nodes of a project
	 *
	 * @param projectname the name of the project
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Required"
	 *
	 **/
	public void importProject(String projectName) throws HeroException {
		log.debug("importProject: project=" + projectName);
		importProject(projectName, "");
	}

	private boolean containsRole(String roleName) {
		BnRoleValue[] rv = mProject.getBnProjectValue().getBnRoles();
		for (int i = 0; i < rv.length; i++) {
			if (rv[i].getName().equalsIgnoreCase(roleName))
				return true;
		}
		return false;
	}

	// -------------------------------------------------------------------------
	// ejbCreate and initProject business methods
	// -------------------------------------------------------------------------

	/**
	 * import all the nodes of a project
	 *
	 * @param projectname the name of the project
	 * @param prefix the prefix of names for the project
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Required"
	 *
	 **/
	public void importProject(String projectName, String prefix) throws HeroException {
		log.debug("importProject: project=" + projectName + " prefix=" + prefix);
		try {
			BnProjectLocalHome opHome = hero.interfaces.BnProjectUtil.getLocalHome();
			BnProjectLocal oldProject = opHome.findByName(projectName);

			//BnProject Hooks
			Collection hooks = oldProject.getBnHooks();
			for (Iterator i = hooks.iterator(); i.hasNext();) {
				BnProjectHookLocal hk = (BnProjectHookLocal) i.next();
				this.addHook(hk.getName(), hk.getEvent(), hk.getType());
			}

			BnRoleValue[] roles = oldProject.getBnProjectValue().getBnRoles();
			for (int r = 0; r < roles.length; r++) {
				String roleName = roles[r].getName();
				if (!containsRole(roleName)) {
					this.addRole(roleName, roles[r].getDescription());
				}
			}

			BnNodeValue[] nodes = oldProject.getBnProjectValue().getBnNodes();
			for (int n = 0; n < nodes.length; n++) {
				String newNodeName = prefix + nodes[n].getName();
				this.addNode(newNodeName, nodes[n].getType());
				this.setNodeRole(newNodeName, nodes[n].getBnRole().getName());
				this.setNodeDescription(newNodeName, nodes[n].getDescription());

				//BnNode Hooks
				BnNodeLocal nd = oldProject.getBnNode(nodes[n].getName());
				Collection nodeHooks = nd.getBnHooks();
				for (Iterator j = hooks.iterator(); j.hasNext();) {
					BnNodeHookLocal nh = (BnNodeHookLocal) j.next();
					this.addNodeHook(newNodeName, nh.getName(), nh.getEvent(), nh.getType());
				}
			}

			BnEdgeValue[] edges = oldProject.getBnProjectValue().getBnEdges();

			for (int e = 0; e < edges.length; e++) {
				this.addEdge(prefix + (edges[e].getInBnNode()).getName(), prefix + (edges[e].getOutBnNode()).getName());
			}

		} catch (FinderException e) {
			throw new HeroException("Cannot find project " + e.getMessage());
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
	}

	/**
	 * init the BnProject Session Bean for SOAP interface only
	 *
	 * @throws HeroException 
	 * @param projectName project name 
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Required"
	 **/
	public void initProject(String projectName) throws HeroException {
		Context lContext;
		BnUserLocalHome uhome = null;
		BnRoleLocalHome rhome = null;
		log.debug("initProject: project=" + projectName);
		try {
			pHome = hero.interfaces.BnProjectUtil.getLocalHome();
			uhome = hero.interfaces.BnUserUtil.getLocalHome();
			rhome = hero.interfaces.BnRoleUtil.getLocalHome();
			this.mProject = pHome.findByName(projectName);		
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			// The project has to be created
			try {
				String creator = mContext.getCallerPrincipal().getName();
				mProject = pHome.create(projectName, creator);
				BnUserLocal user = uhome.findByName(creator);
				mProject.getBnUsers().add(user);
				log.debug("Add user " + user.getBnUserLightValue().toString());
				// role admin is added
				BnRoleValue rv = new BnRoleValue();
				rv.setName(hero.interfaces.Constants.ADMIN);
				rv.setDescription("the admin role");
				BnRoleLocal role = rhome.create(rv);
				mProject.getBnRoles().add(role);
				this.setUserRole(creator, hero.interfaces.Constants.ADMIN);
				// role InitialRole is added
				rv = new BnRoleValue();
				rv.setName(hero.interfaces.Constants.INITIALROLE);
				rv.setDescription("the initial role for nodes");
				role = rhome.create(rv);
				mProject.getBnRoles().add(role);
				jms.sendProjectEvent(ADDUSERPROJECT, projectName, creator);
			} catch (FinderException fie) {
				throw new HeroException(fie.getMessage());
			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			} catch (Exception he) {
				throw new HeroException(he.getMessage());
			}
		}
	}

	/**
	 * init the BnProject Session Bean for SOAP interface only (clone project)
	 *
	 * @throws HeroException 
	 * @param oldProject name
	 * @param newProject name 
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Required"
	 **/
	public void initProject(String oldProject, String newProject) throws CreateException {
		log.debug("initProject: oldProject=" + oldProject + " newProject=" + newProject);
		Context lContext;
		BnUserLocalHome uhome = null;
		BnRoleLocalHome rhome = null;

		try {
			pHome = hero.interfaces.BnProjectUtil.getLocalHome();
			uhome = hero.interfaces.BnUserUtil.getLocalHome();
			rhome = hero.interfaces.BnRoleUtil.getLocalHome();
			mProject = pHome.findByName(newProject);
		} catch (javax.naming.NamingException ne) {
			throw new CreateException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			// The project has to be created
			try {
				String creator = mContext.getCallerPrincipal().getName();
				mProject = pHome.create(newProject, creator);
				BnUserLocal user = uhome.findByName(creator);
				mProject.getBnUsers().add(user);
				log.debug("Add user " + user.getBnUserLightValue().toString());
				// role admin is added
				BnRoleValue rv = new BnRoleValue();
				rv.setName(hero.interfaces.Constants.ADMIN);
				rv.setDescription("the admin role");
				BnRoleLocal role = rhome.create(rv);
				mProject.getBnRoles().add(role);
				this.setUserRole(creator, hero.interfaces.Constants.ADMIN);
				// role InitialRole is added
				rv = new BnRoleValue();
				rv.setName(hero.interfaces.Constants.INITIALROLE);
				rv.setDescription("the initial role for nodes");
				role = rhome.create(rv);
				mProject.getBnRoles().add(role);
				importProject(oldProject);

				jms.sendProjectEvent(ADDUSERPROJECT, newProject, creator);

			} catch (FinderException fie) {
				throw new CreateException(fie.getMessage());
			} catch (InvalidValueException ie) {
				throw new CreateException(ie.getMessage());
			} catch (HeroException he) {
				throw new CreateException(he.getMessage());
			}
		}

	}

	/**
	 * Create the BnProject Session Bean for SOAP interface only
	 *
	 * @throws CreateException 
	 *
	 * @ejb:create-method view-type="both"
	 **/
	public void ejbCreate() throws CreateException {
	}

	public void ejbPostCreate(String oldProject, String newProject) throws CreateException, RemoteException {
	}

	public void setSessionContext(final javax.ejb.SessionContext context) {
		mContext = context;
		jms = new JMSEvents();
	}

	public void ejbRemove() {
	}

	public void ejbActivate() {
	}

	public void ejbPassivate() {
	}

	// Utility methods

	private boolean checkDynamics(String out) throws HeroException {
		BnNodeLocal nodeout = mProject.getBnNode(out);
		if ((nodeout.getState() == hero.interfaces.Constants.Nd.INITIAL)
			|| (nodeout.getState() == hero.interfaces.Constants.Nd.READY)
			|| (nodeout.getState() == hero.interfaces.Constants.Nd.ANTICIPABLE))
			return true;
		return false;
	}

	private boolean checkStructuralConstraints() throws HeroException {
		BnProjectValue pv = mProject.getBnProjectValue();
		return true;
	}

	private boolean checkDuplicateEdge(String in, String out) throws HeroException {
		Collection edges = mProject.getBnEdges();
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnEdgeLocal edge = (BnEdgeLocal) i.next();
			if ((edge.getInBnNode().getName()).equals(in) && (edge.getOutBnNode().getName()).equals(out))
				return false;
		}
		return true;
	}

	private boolean checkDuplicateAgentEdge(String in, String out) throws HeroException {
		Collection edges = mProject.getBnAgentEdges();
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnAgentEdgeLocal edge = (BnAgentEdgeLocal) i.next();
			if ((edge.getInBnAgent().getName()).equals(in) && (edge.getOutBnNode().getName()).equals(out))
				return false;
		}
		return true;
	}
	
        /**
	 * Test if the project contains a user
	 * @return true is the users belongs to the project
	 * @param userName The name of the user to associate to the project
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/
	public boolean containsUser(String userName) throws HeroException {
	log.debug("containsUser : name=" + userName);
	BnUserLocalHome uHome;
	BnUserLocal mUser;
	
	try {
		uHome = (BnUserLocalHome) BnUserUtil.getLocalHome();
	} catch (javax.naming.NamingException be) {
		throw new HeroException(be.getMessage());
	}
	try {
		mUser = uHome.findByName(userName);
	} catch (javax.ejb.FinderException fe) {
		throw new HeroException("BnUser does not exists");
	}
	Collection cUsers = mProject.getBnUsers();
	return (cUsers.contains(mUser));
	}
	
}
