package hero.session;
/*
*
* ProjectSoapXMLBean.java - 
* Copyright (C) 2003 Ecoo Team
* valdes@loria.fr
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import hero.interfaces.Constants;
import hero.entity.NodeState;
import hero.interfaces.BnEdgeLocal;
import hero.interfaces.BnEdgeLocalHome;
import hero.interfaces.BnEdgeUtil;
import hero.interfaces.BnEdgeValue;
import hero.interfaces.BnNodeHookLocal;
import hero.interfaces.BnNodeHookLocalHome;
import hero.interfaces.BnNodeHookUtil;
import hero.interfaces.BnNodeHookValue;
import hero.interfaces.BnNodeInterHookLocal;
import hero.interfaces.BnNodeInterHookLocalHome;
import hero.interfaces.BnProjectUtil;
import hero.interfaces.BnNodeInterHookUtil;
import hero.interfaces.BnNodeLightValue;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodeLocalHome;
import hero.interfaces.BnNodePropertyLocal;
import hero.interfaces.BnNodePropertyLocalHome;
import hero.interfaces.BnNodePropertyUtil;
import hero.interfaces.BnNodePropertyValue;
import hero.interfaces.BnNodeUtil;
import hero.interfaces.BnNodeValue;
import hero.interfaces.BnProjectHookLocal;
import hero.interfaces.BnProjectHookLocalHome;
import hero.interfaces.BnProjectHookUtil;
import hero.interfaces.BnProjectHookValue;
import hero.interfaces.BnProjectInterHookLocal;
import hero.interfaces.BnProjectInterHookLocalHome;
import hero.interfaces.BnProjectInterHookUtil;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectPropertyLocal;
import hero.interfaces.BnProjectPropertyLocalHome;
import hero.interfaces.BnProjectPropertyUtil;
import hero.interfaces.BnProjectPropertyValue;
import hero.interfaces.BnProjectValue;
import hero.interfaces.BnRoleLocal;
import hero.interfaces.BnRoleLocalHome;
import hero.interfaces.BnRoleUtil;
import hero.interfaces.BnRoleValue;
import hero.interfaces.BnUserLightValue;
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.interfaces.BnUserUtil;
import hero.util.EventConstants;
import hero.util.JMSEvents;
import hero.util.NodeExecutingException;
import hero.util.DeleteEdgeException;
import hero.util.DuplicatedEdgeException;
import hero.util.EdgeCycleException;
import hero.util.HeroException;
import hero.interfaces.InvalidValueException;
//import hero.util.InvalidValueException;
import hero.util.NodeStartedException;
import hero.util.NodeTerminatedException;


import hero.util.NodeHooks;
import hero.util.NodeInEdges;
import hero.util.NodeOutEdges;
import hero.util.NodeProperties;
import hero.util.ProjectHooks;
import hero.util.ProjectNodes;
import hero.util.ProjectProperties;
import hero.util.ProjectRoles;
import hero.util.ProjectUsers;
import hero.util.TimerFilter;
import hero.hook.Hook;

import java.io.StringWriter;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.NotificationFilter;
import javax.management.ObjectName;
import javax.naming.Context;

import org.apache.log4j.Category;
import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.ValidationException;
import hero.util.StrutsNodeValue;


/**
 * Session Bean Template
 *
 * @ejb:bean name="ProjectSoapXML"
 *           display-name="ProjectSoapXML Bean"
 *           type="Stateful"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/ProjectSoapXML"
 *
 * @ejb:ejb-ref	ejb-name="ProjectSoapXML"
 *              	view-type="remote"
 *              	ref-name="ProjectSoapXML"
 *
 * @ejb:ejb-ref ejb-name="BnProject"
 *              ref-name="myhero/BnProject"
 *
 * @ejb:transaction type="Supports"
 * @ejb.permission role-name="Admin,user"
 * 
 * @jonas.bean
 *      ejb-name="ProjectSoapXML"
 *      jndi-name="ejb/hero/ProjectSoapXML"
 **/

public class ProjectSoapXMLBean implements SessionBean, EventConstants {

	// -------------------------------------------------------------------------
	// Static
	// -------------------------------------------------------------------------
	// Utility variable
	private static final Category log = Category.getInstance(ProjectSoapXMLBean.class);

	// -------------------------------------------------------------------------
	// Members 
	// -------------------------------------------------------------------------

	private SessionContext mContext;

	private BnProjectLocalHome pHome;
	private BnProjectLocal mProject;
	private JMSEvents jms;

	// -------------------------------------------------------------------------
	// Add and Set Methods
	// -------------------------------------------------------------------------

	/**
	* Add a BnNode to the BnProject
	*
	* @param name The name of the node to add to the project
	* @param nodeType The type of the node to add to the project
	*
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	*
	**/
	public void addNode(String name, int nodeType) throws HeroException {
		log.debug("addNode: name=" + name + " nodeType:");
		BnNodeLocalHome nHome;
		BnNodeLocal mNode;
		try {
			nHome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			BnNodeLocal nd = nHome.create(name, nodeType);
			BnNodeValue ndata = nd.getBnNodeValue();
			ndata.setCreationDate(new java.sql.Date(new java.util.Date().getTime()));
			nd.setBnNodeValue(ndata);
			Collection cNodes = mProject.getBnNodes();
			cNodes.add(nd);
			String creator = mContext.getCallerPrincipal().getName();
			BnUserLocalHome uHome = BnUserUtil.getLocalHome();
			BnUserLocal user = uHome.findByName(creator);
			nd.setCreator(user);
			BnRoleLocalHome rHome = BnRoleUtil.getLocalHome();
			BnRoleLocal role = rHome.findByName(Constants.INITIALROLE, mProject.getId());
			nd.setBnRole(role);

			// Tell the world
			jms.sendNodeEvent(ADDNODE, this.getDetails().getName(), name, nodeType, ndata.getState(), mContext.getCallerPrincipal().getName());

		} catch (InvalidValueException ie) {
			throw new HeroException(ie.getMessage());
		} catch (CreateException ce) {
			throw new HeroException(ce.getMessage());
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
	}

	/**
	* Add an BnEdge to the BnProject [in] ---- [out]
	*
	* @param in the name of the in BnNode
	* @param out the name of the out BnNode
	*
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	*
	**/
	public String addEdge(String in, String out) throws HeroException {
		log.debug("addEdge: in=" + in + " out:" + out);
		BnEdgeLocalHome eHome;
		BnNodeLocalHome nHome;
		BnNodeLocal mInNode;
		BnNodeLocal mOutNode;
		BnEdgeLocal mEdge;
		try {
			nHome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			mInNode = nHome.findByName(in, mProject.getId());
		} catch (FinderException fe) {
			throw new HeroException("In BnNode does not exist");
		}
		try {
			mOutNode = nHome.findByName(out, mProject.getId());
		} catch (FinderException fe) {
			throw new HeroException("Out BnNode does not exist");
		}
		try {
			eHome = BnEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		if (checkDynamics(in, out)) { // check state of Nodes
			if (checkDuplicateEdge(in, out)) {
				try {
					mEdge = eHome.create();
					mEdge.setBnProject(mProject);
					mEdge.setInBnNode(mInNode);
					mEdge.setOutBnNode(mOutNode);

					jms.sendEdgeEvent(
						ADDEDGE,
						this.getDetails().getName(),
						mEdge.getName(),
						mInNode.getName(),
						mOutNode.getName(),
						mContext.getCallerPrincipal().getName());

				} catch (InvalidValueException ie) {
					throw new HeroException(ie.getMessage());
				} catch (CreateException ce) {
					throw new HeroException(ce.getMessage());
				}
			} else {
				throw new DuplicatedEdgeException("Cannot add edge. BnEdge between nodes already exist");
			}
		} else {
			throw new NodeStartedException("Cannot add edge. " + out + " is already started");
		}
		if (!checkStructuralConstraints())
			throw new EdgeCycleException("This edge add a cycle");

		// update the current graph state
		int edgeState = Constants.edgeTransition[Constants.Ed.CONDITION_TRUE][mInNode.getState()];
		this.setEdgeState(mEdge, edgeState);
		//	mEdge.setState(edgeState);

		int newOperation = mOutNode.getActivation().state(mOutNode);
		int newState = mOutNode.getTransition().computeState(mOutNode, newOperation);
		if (newState == Constants.Nd.BAD_TRANSITION) {
			throw new HeroException("AddEdge generated a bad BnNode transition");
		}
		this.setNodeState(mOutNode, newState);
		//	jms.sendNodeEvent(READYINITIAL,this.getName(),nOut.getName(),nOut.getType(),INITIAL,context.getCallerPrincipal().getName());

		return mEdge.getName();
	}

	/**
	* add an BnEdge to the BnProject
	*
	* @param name the name ot the edge
	*
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	*
	**/
	public void addEdge(String name, String in, String out) throws HeroException {
		log.debug("addEdge: name=" + name + " in=" + in + " out=" + out);
		throw new HeroException("no more implemented");
	}

	/**
	* Add a BnUser to the BnProject
	*
	* @param userName The name of the user to associate to the project
	*
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	*
	**/
	public void addUser(String userName) throws HeroException {
		log.debug("addUser: name=" + userName);
		BnUserLocalHome uHome;
		BnRoleLocalHome rHome;
		BnUserLocal mUser;

		try {
			uHome = BnUserUtil.getLocalHome();
			rHome = BnRoleUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			mUser = uHome.findByName(userName);
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException("BnUser does not exist");
		}

		Collection cUsers = mProject.getBnUsers();
		cUsers.add(mUser);
		this.addRole(userName, "BnUser BnRole");
		this.setUserRole(userName, userName);
		jms.sendProjectEvent(ADDUSERPROJECT, this.getDetails().getName(), userName);

	}

	/**
	* Add a role to the project
	*
	* @param name the name of the role
	* @param description the description of the role
	*
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	*
	**/
	public void addRole(String roleName, String description) throws HeroException {
		BnRoleLocalHome roleh;
		log.debug("addRole: roleName=" + roleName + " description=" + description);
		try {
			roleh = BnRoleUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		BnRoleValue rv = new BnRoleValue();
		rv.setName(roleName);
		rv.setDescription(description);
		try {
			BnRoleLocal rl = roleh.create(rv);
			mProject.getBnRoles().add(rl);
			jms.sendRoleEvent(ADDROLE, this.getDetails().getName(), roleName);
		} catch (javax.ejb.CreateException ce) {
			throw new HeroException(ce.getMessage());
		}
	}

	/**
	 * add hook to a node
	 *
	 * @param nodeName the name of the node
	 * @param hookName the name of the hook
	 * @param eventName the name of the event
	 * @param hookType the type of the hook
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Required"
	 *
	 **/
	public void addNodeHook(String nodeName, String hookName, String eventName, int hookType) throws HeroException {
		log.debug("addNodeHook: node=" + nodeName + " hookName=" + hookName + " eventName=" + eventName);
		BnNodeHookLocalHome pHome;
		BnNodeHookLocal ppLocal;

		BnNodeLocal nd = mProject.getBnNode(nodeName);
		try {
			pHome = BnNodeHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}

		try {
			BnNodeHookLocal nf = pHome.findByName(nd.getId(), hookName);
			throw new HeroException("BnNode hook " + hookName + " already exist in node " + nodeName);
		} catch (javax.ejb.FinderException nn) {

			try {
				BnNodeHookLocal h = pHome.create(hookName, eventName, hookType);
				h.setBnNode(nd);
			} catch (CreateException fe) {
				throw new HeroException(fe.getMessage());
			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			}
		}
	}

	/**
	 * add hook to project
	 *
	 * @param hookName the name of the hook
	 * @param eventName the event
	 * @param hookType the type
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Required"
	 *
	 **/
	public void addHook(String hookName, String eventName, int hookType) throws HeroException {
		log.debug("addHook: hookName=" + hookName + " eventName=" + eventName);
		BnProjectHookLocalHome pHome;
		BnProjectHookLocal ppLocal;

		try {
			pHome = BnProjectHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}

		try {
			BnProjectHookLocal pf = pHome.findByName(mProject.getId(), hookName);
			throw new HeroException("BnProject hook " + hookName + " already exist in project " + mProject.getName());
		} catch (javax.ejb.FinderException pn) {

			try {
				BnProjectHookLocal h = pHome.create(hookName, eventName, hookType);
				h.setBnProject(mProject);
			} catch (CreateException fe) {
				throw new HeroException(fe.getMessage());
			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			}
		}
	}

	/**
	 * add interhook to a node
	 *
	 * @param nodeName the name of the node
	 * @param hookName the name of the hook
	 * @param eventName the name of the event
	 * @param hookType the type of the hook
	 * @param script the value of the script 
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Required"
	 *
	 **/
	public void addNodeInterHook(String nodeName, String hookName, String eventName, int hookType, String value) throws HeroException {
		log.debug("addNodeHook: node=" + nodeName + " hookName=" + hookName + " eventName=" + eventName + " value=" + value);
		BnNodeInterHookLocalHome pHome;
		BnNodeInterHookLocal ppLocal;

		BnNodeLocal nd = mProject.getBnNode(nodeName);
		try {
			pHome = BnNodeInterHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}

		try {
			BnNodeInterHookLocal nf = pHome.findByName(nd.getId(), hookName);
			throw new HeroException("Interactive node hook " + hookName + " already exist in node " + nodeName);
		} catch (javax.ejb.FinderException nn) {
			try {
				BnNodeInterHookLocal h = pHome.create(hookName, eventName, hookType, value);
				h.setBnNode(nd);
			} catch (CreateException fe) {
				throw new HeroException(fe.getMessage());
			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			}
		}
	}

	/**
	 * add interhook to the project
	 *
	 * @param hookName the name of the hook
	 * @param eventName the name of the event
	 * @param hookType the type of the hook
	 * @param script the value of the script 
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Required"
	 *
	 **/
	public void addInterHook(String hookName, String eventName, int hookType, String value) throws HeroException {
		log.debug("addNodeHook: hookName=" + hookName + " eventName=" + eventName + " value=" + value);
		BnProjectInterHookLocalHome pHome;
		BnProjectInterHookLocal ppLocal;

		try {
			pHome = BnProjectInterHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}

		try {
			BnProjectInterHookLocal nf = pHome.findByName(mProject.getId(), hookName);
			throw new HeroException("Interactive hook " + hookName + " already exist in project " + mProject.getName());
		} catch (javax.ejb.FinderException nn) {
			try {
				BnProjectInterHookLocal h = pHome.create(hookName, eventName, hookType, value);
				h.setBnProject(mProject);
			} catch (CreateException fe) {
				throw new HeroException(fe.getMessage());
			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			}
		}
	}

	/**
	 * Set the node hook value. Only for interactives hooks
	 *
	 * @param name of the node
	 * @param description of the node
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Required"
	 *
	 **/
	public void setNodeInterHookValue(String node, String hook, String value) throws HeroException {
		log.debug("setNodeInterHookValue: node=" + node + " hook=" + hook + " value=" + value);
		BnNodeLocal nd = mProject.getBnNode(node);
		Collection hooks = nd.getBnInterHooks();
		for (Iterator ih = hooks.iterator(); ih.hasNext();) {
			BnNodeInterHookLocal hk = (BnNodeInterHookLocal) ih.next();
			if (hk.getName().equals(hook)) {
				hk.setScript(value);
			}
		}
	}

	/**
	 * Set the hook value. Only for interactives hooks
	 *
	 * @param description of the node
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Required"
	 *
	 **/
	public void setInterHookValue(String hook, String value) throws HeroException {
		log.debug("setNodeInterHookValue:  hook=" + hook + " value=" + value);
		Collection hooks = mProject.getBnInterHooks();
		for (Iterator ih = hooks.iterator(); ih.hasNext();) {
			BnProjectInterHookLocal hk = (BnProjectInterHookLocal) ih.next();
			if (hk.getName().equals(hook)) {
				hk.setScript(value);
			}
		}
	}
	
	/**
	 * Set edit node changes. Only for GraphEditor client
	 *
	 * @param name of the node
	 * @param role of the node
	 * @param description of the node
	 * @param deadline of the node
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Required"
	 *
	 **/
	public void setEditNode(String node, String role, String description, long deadline) throws HeroException {
		log.debug("setEditNode:  node=" + node + " role=" + role +" description=" + description + " deadline=" + deadline);
		this.setNodeDeadline(node,deadline);
		this.setNodeDescription(node,description);
		this.setNodeRole(node,role);		
	}

	/**
	* delete a node from the BnProject
	*
	* @param name the name ot the edge
	*
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	*
	**/
	public void deleteNode(String name) throws HeroException {
		// check deletion conditions
		log.debug("deleteNode: name=" + name);

		BnNodeLocal nd = mProject.getBnNode(name);
		if (nd.isExecuting()) {
			throw new NodeExecutingException("Cannot delete node. It is executing");
		}
		if (nd.isTerminated()) {
			throw new NodeTerminatedException("Cannot delete node. It is terminated");
		}
		Collection edgesIn = nd.getInBnEdges();
		Object[] edai = edgesIn.toArray();
		for (int i = 0; i < edai.length; i++) {
			edgeDeletion((BnEdgeLocal) edai[i]);
		}
		Collection edgesOut = nd.getOutBnEdges();
		Object[] edao = edgesOut.toArray();
		for (int i = 0; i < edao.length; i++) {
			edgeDeletion((BnEdgeLocal) edao[i]);
		}
		try {
			int state = nd.getState();
			int type = nd.getType();
			nd.remove();
			jms.sendNodeEvent(DELETENODE, this.getDetails().getName(), name, type, state, mContext.getCallerPrincipal().getName());
			removeOldNotification(name + ":" + this.getDetails().getName());
		} catch (RemoveException re) {
			throw new HeroException("Cannot remove node " + re.getMessage());
		}
		log.debug("deleteNode -end: name=" + name);
	}

	/**
	* delete an BnEdge to the BnProject
	*
	* @param name the name ot the edge
	*
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	*
	**/
	public void deleteEdge(String name) throws HeroException {
		// check deletion conditions
		log.debug("deleteEdge: name=" + name);
		BnEdgeLocalHome eh;
		BnEdgeLocal ed;
		try {
			eh = BnEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			ed = eh.findByName(name, mProject.getId());
		} catch (FinderException fe) {
			throw new HeroException("edge does not exists " + name);
		}
		jms.sendEdgeEvent(
			DELETEEDGE,
			this.getDetails().getName(),
			name,
			ed.getInBnNode().getName(),
			ed.getOutBnNode().getName(),
			mContext.getCallerPrincipal().getName());
		edgeDeletion(ed);
		log.debug("deleteEdge -end: name=" + name);

	}

	/**
	 * detete a hook
	 *
	 * @param hookName the name of the hook
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Required"
	 *
	 **/
	public void deleteHook(String hookName) throws HeroException {
		BnProjectHookLocalHome pHome;
		BnProjectHookLocal ppLocal;

		try {
			pHome = BnProjectHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}

		try {
			BnProjectHookLocal ph = pHome.findByName(mProject.getId(), hookName);
			ph.remove();
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		} catch (RemoveException rm) {
			throw new HeroException(rm.getMessage());
		}
	}

	/**
	 * remove a node hook
	 *
	 * @param nodeName the name of the node
	 * @param hookName the name 
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Required"
	 *
	 **/
	public void deleteNodeHook(String nodeName, String hookName) throws HeroException {
		BnNodeHookLocalHome pHome;
		BnNodeHookLocal ppLocal;
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		try {
			pHome = BnNodeHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			BnNodeHookLocal nh = pHome.findByName(nd.getId(), hookName);
			nh.remove();
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		} catch (RemoveException rm) {
			throw new HeroException(rm.getMessage());
		}
	}

	/**
	* Add a role to the project
	*
	* @param userName the name of the user
	* @param roleName the name of the role
	*
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	*
	**/
	public void setUserRole(String userName, String roleName) throws HeroException {
		log.debug("setUserRole: user=" + userName + " role=" + roleName);
		BnRoleLocalHome roleh;
		BnUserLocalHome userh;
		try {
			roleh = BnRoleUtil.getLocalHome();
			userh = BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			BnUserLocal user = userh.findByName(userName);
			BnRoleLocal role = roleh.findByName(roleName, mProject.getId());
			user.getBnRoles().add(role);
			jms.sendUserRoleEvent(SETUSERROLE, this.getDetails().getName(), userName, roleName);
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
	}

	/**
	* Set the node State
	*
	* @param name of the node
	* @param new state of the node
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	*
	**/
	public void setNodeState(BnNodeLocal node, int state) throws HeroException {
		log.debug("setNodeState: name=" + node.getName() + " state=" + state);
		node.setState(state);
		if (state == Constants.Nd.EXECUTING) {
			node.setStartDate(new java.sql.Date(new java.util.Date().getTime()));
		}
		if (state == Constants.Nd.TERMINATED) {
			node.setEndDate(new java.sql.Date(new java.util.Date().getTime()));
			this.propagateNodeProperties(node.getName());
		}
		jms.sendNodeEvent(SETNODESTATE, this.getDetails().getName(), node.getName(), node.getType(), state, mContext.getCallerPrincipal().getName());

	}

	/**
	* Set the edge State
	*
	* @param bean of the edge
	* @param new state of the edge
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	*
	**/
	public void setEdgeState(BnEdgeLocal edge, int state) throws HeroException {
		log.debug("setEdgeState: name=" + edge.getName() + " state=" + state);
		edge.setState(state);
		jms.sendEdgeEvent(
			SETEDGESTATE,
			this.getDetails().getName(),
			edge.getName(),
			edge.getInBnNode().getName(),
			edge.getOutBnNode().getName(),
			mContext.getCallerPrincipal().getName());
	}

	private void removeOldNotification(String name) throws HeroException {
		try {
			ObjectName timer = new ObjectName("bonita.jmx:service=timer");
			List list = MBeanServerFactory.findMBeanServer(null);
			MBeanServer server = (MBeanServer) list.iterator().next();

			server.invoke(timer, "removeNotifications", new Object[] { name }, new String[] { "".getClass().getName()});
		} catch (javax.management.InstanceNotFoundException ex) {
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}

	/**
	* Set the node deadline
	*
	* @param name of the node
	* @param Date of the deadline
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	*
	**/
	public void setNodeDeadline(String name, long date) throws HeroException {
			log.debug("setBnNodeDeadline: name=" + name + " deadline=" + date);

		
			BnNodeLocal nd = mProject.getBnNode(name);
			nd.setDeadline(new java.sql.Date(date));
			String projectName = this.getDetails().getName();
			jms.sendNodeEvent(SETNODEDEADLINE, projectName, name, nd.getType(), nd.getState(), mContext.getCallerPrincipal().getName());

			try {
				String type = name + ":" + projectName;
				removeOldNotification(type);

/*				ObjectName timer = new ObjectName("jboss:service=timer");
				ObjectName receiver = new ObjectName("jboss:service=TimerReceiver");*/
				ObjectName timer    = new ObjectName("jonas:type=service,name=Timer");
				ObjectName receiver    = new ObjectName("jonas:type=service,name=TimerReceiver");
 				List list = MBeanServerFactory.findMBeanServer(null);
				MBeanServer server = (MBeanServer) list.iterator().next();
				Date lNext = new Date(date);
				NotificationFilter filter = new TimerFilter();
				server.addNotificationListener(timer, receiver, filter, null);
				Integer id = (Integer) server.invoke(timer, "addNotification", new Object[] { type, //type
					name, //message
					"bonita", //user object
					lNext, //delay time
				}, new String[] { "".getClass().getName(), "".getClass().getName(), "java.lang.Object", Date.class.getName()});
			} catch (Exception e) {e.printStackTrace();
				throw new HeroException(e.getMessage());
			}

		}
	/**
	* Set the node description
	*
	* @param name of the node
	* @param description of the node
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	*
	**/
	public void setNodeDescription(String name, String description) throws HeroException {
		log.debug("setNodeDescription: name=" + name + " description=" + description);
		BnNodeLocal nd = mProject.getBnNode(name);
		nd.setDescription(description);
	}

	/**
	* Set the node in traditional mode
	*
	* @param name of the node
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	*
	**/
	public void setNodeTraditional(String name) throws HeroException {
		log.debug("setNodeTraditional: name=" + name);
		BnNodeLocal nd = mProject.getBnNode(name);
		nd.setAnticipable(false);
		nd.setTransition(NodeState.make(nd.getType(), false));

	}

	/**
	* Set the node in anticipable mode
	*
	* @param name of the node
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	*
	**/
	public void setNodeAnticipable(String name) throws HeroException {
		log.debug("setNodeAnticipable: name=" + name);
		BnNodeLocal nd = mProject.getBnNode(name);
		nd.setAnticipable(true);
		nd.setTransition(NodeState.make(nd.getType(), true));
	}

	/**
	* Set the name of the node executor
	*
	* @param name the name of the node
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	*
	**/
	public void setNodeExecutor(String name) throws HeroException {
		log.debug("setNodeExecutor: name=" + name);
		BnUserLocalHome uHome;
		BnUserLocal user;
		try {
			uHome = BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		BnNodeLocal nd = mProject.getBnNode(name);
		String executor = mContext.getCallerPrincipal().getName();
		try {
			user = uHome.findByName(executor);
		} catch (FinderException fe) {
			throw new HeroException("BnUser " + executor + " does not exists");
		}
		nd.setExecutor(user);
		jms.sendNodeEvent(SETNODEEXECUTOR, this.getDetails().getName(), name, nd.getType(), nd.getState(), mContext.getCallerPrincipal().getName());
	}
	/**
	 * Set a property of a node
	 *
	 * @param nodeName the name of the node
	 * @param key the key of the property
	 * @param value the name of the property
	 * @param propagate if it is a propagation property
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Required"
	 *
	 **/
	public void setNodeProperty(String nodeName, String key, String value, boolean propagate) throws HeroException {

		BnNodePropertyLocalHome pHome;
		BnNodePropertyLocal propertyLocal;

		log.debug("setNodeProperty node=" + nodeName + " key=" + key + " value=" + value);

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		try {
			pHome = BnNodePropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(nd.getId(), key);
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
		if (!c.isEmpty()) {
			propertyLocal = (BnNodePropertyLocal) (c.toArray())[0];
			propertyLocal.setTheValue(value);
			propertyLocal.setPropagate(propagate);
		} else {
			try {
				propertyLocal = pHome.create(key, value);
				propertyLocal.setBnNode(nd);
				propertyLocal.setPropagate(propagate);
			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			} catch (javax.ejb.CreateException ce) {
				throw new HeroException(ce.getMessage());
			}

		}

	}

	/**
	 * Set a property of a node
	 *
	 * @param nodeName the name of the node
	 * @param key the key of the property
	 * @param value the name of the property
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Required"
	 *
	 **/

	public void setNodeProperty(String nodeName, String key, String value) throws HeroException {
		BnNodePropertyLocalHome pHome;
		BnNodePropertyLocal propertyLocal = null;

		log.debug("setNodeProperty node=" + nodeName + " key=" + key + " value=" + value);

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		try {
			pHome = BnNodePropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(nd.getId(), key);
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}

		if (!c.isEmpty()) {

			propertyLocal = (BnNodePropertyLocal) (c.toArray())[0];
			propertyLocal.setTheValue(value);

		} else {
			try {
				propertyLocal = pHome.create(key, value);
				propertyLocal.setBnNode(nd);

			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			} catch (javax.ejb.CreateException ce) {
				throw new HeroException(ce.getMessage());
			}

		}

	}

	/**
	  * Delete a property of a node
	  *
	  * @param nodeName the name of the node
	  * @param key the key of the property
	  * @ejb:interface-method view-type="both"
          * @ejb:transaction type="Required"
	  *
	  **/
	public void deleteNodeProperty(String nodeName, String key) throws HeroException {
		log.debug("deleteNodeProperty node=" + nodeName + " key=" + key);

		BnNodePropertyLocalHome pHome;
		BnNodePropertyLocal propertyLocal = null;

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		try {
			pHome = BnNodePropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(nd.getId(), key);
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}

		if (!c.isEmpty()) {
			try {
				propertyLocal = (BnNodePropertyLocal) (c.toArray())[0];
				propertyLocal.remove();
			} catch (RemoveException rm) {
				throw new HeroException(rm.getMessage());
			}

		} else {
			throw new HeroException("Property not found :" + key);
		}
	}

	/**
	 * Set a property of the project
	 *
	 * @param key the key of the property
	 * @param value the name of the property
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Required"
	 *
	 **/
	public void setProperty(String key, String value) throws HeroException {
		log.debug("setProperty key=" + key + " value=" + value);

		BnProjectPropertyLocalHome pHome;
		BnProjectPropertyLocal ppLocal = null;

		try {
			pHome = BnProjectPropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(mProject.getId(), key);
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}

		if (!c.isEmpty()) {

			ppLocal = (BnProjectPropertyLocal) (c.toArray())[0];
			ppLocal.setTheValue(value);

		} else {
			try {
				ppLocal = pHome.create(key, value);
				ppLocal.setBnProject(mProject);

			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			} catch (javax.ejb.CreateException ce) {
				throw new HeroException(ce.getMessage());
			}

		}

	}

	/**
	  * Delete a property of the project
	  *
	  * @param key the key of the property
	  * @ejb:interface-method view-type="both"
          * @ejb:transaction type="Required"
	  *
	  **/
	public void deleteProperty(String key) throws HeroException {
		log.debug("deleteProperty key=" + key);

		BnProjectPropertyLocalHome pHome;
		BnProjectPropertyLocal ppLocal;

		try {
			pHome = BnProjectPropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(mProject.getId(), key);
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}

		if (!c.isEmpty()) {
			try {
				ppLocal = (BnProjectPropertyLocal) (c.toArray())[0];
				ppLocal.remove();
			} catch (RemoveException rm) {
				throw new HeroException(rm.getMessage());
			}

		} else {
			throw new HeroException("Property not found :" + key);
		}
	}

	/**
	   * Propagate BnNode properties 
	   * @param nodeName the name of the BnNode
	   * @ejb:interface-method view-type="both"
           * @ejb:transaction type="Required"
	   *
	   **/
	public void propagateNodeProperties(String nodeName) throws HeroException {
		log.debug("propagateNodeProperties: name=" + nodeName);

		BnNodePropertyLocalHome pHome;
		BnNodePropertyLocal propertyLocal = null;

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		Collection c = nd.getBnProperties();
		Collection c2 = nd.getOutBnEdges();
		if ((!c.isEmpty()) && (!c2.isEmpty())) {
			BnNodePropertyLocal p;
			BnEdgeLocal ed;
			for (Iterator i = c.iterator(); i.hasNext();) {
				p = (BnNodePropertyLocal) i.next();
				if (p.getPropagate()) {
					for (Iterator i2 = c2.iterator(); i2.hasNext();) {
						ed = (BnEdgeLocal) i2.next();
						this.setNodeProperty((ed.getOutBnNode()).getName(), p.getTheKey(), p.getTheValue());
					}
				}
			}
		}
	}

	/**
	* Sets the role of an activity
	*
	* @param name the name of the activity
	* @param role the name of the role
	*
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	*
	**/
	public void setNodeRole(String name, String role) throws HeroException {
		log.debug("setNodeRole: name=" + name + " role=" + role);
		BnRoleLocalHome rh;
		BnNodeLocalHome nh;
		BnNodeLocal nl;
		BnRoleLocal rl;
		try {
			rh = BnRoleUtil.getLocalHome();
			nh = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			nl = nh.findByName(name, mProject.getId());
		} catch (FinderException fe) {
			throw new HeroException("BnNode does not exists " + name);
		}
		try {
			rl = rh.findByName(role, mProject.getId());
		} catch (FinderException fe) {
			throw new HeroException("BnRole does not exists " + role);
		}
		nl.setBnRole(rl);
		jms.sendProjectEvent(CHANGEROLE, this.getDetails().getName());
	}

	/**
	 * Execute a hook
	 *
	 * @param nodeName the name of the node
	 * @param event the event 
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Required"
	 *
	 **/
	public void hookEvent(String nodeName, String event) throws HeroException {
		log.debug("hookEvent: node=" + nodeName + " event=" + event);
		Collection hooks;
		BnNodeLocal node;

		node = mProject.getBnNode(nodeName);
		hooks = node.getBnHooks();
		for (Iterator i = hooks.iterator(); i.hasNext();) {
			BnNodeHookLocal hk = (BnNodeHookLocal) i.next();
			if (hk.getEvent().equalsIgnoreCase(event)) {
				Hook h = Hook.make(hk.getName(), event, hk.getType());
				h.execute(this, event, node);
			}
		}
	}

	/**
	* Test if the project contains a user
	* @return true is the users belongs to the project
	* @param userName The name of the user to associate to the project
	*
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Supports"
	*
	**/
	public boolean containsUser(String userName) throws HeroException {
		log.debug("containsUser : name=" + userName);
		BnUserLocalHome uHome;
		BnUserLocal mUser;

		try {
			uHome = (BnUserLocalHome) BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			mUser = uHome.findByName(userName);
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException("BnUser does not exists");
		}
		Collection cUsers = mProject.getBnUsers();
		return (cUsers.contains(mUser));
	}

	private boolean containsRole(String roleName) {
		BnRoleValue[] rv = mProject.getBnProjectValue().getBnRoles();
		for (int i = 0; i < rv.length; i++) {
			if (rv[i].getName().equalsIgnoreCase(roleName))
				return true;
		}
		return false;
	}

	// -------------------------------------------------------------------------
	// XML Get Methods (Castor Data Binding)
	// -------------------------------------------------------------------------

	/**
	 * Get project information
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getProjectDetails() throws HeroException {

		try {
			//BnProjectLightValue pl = mProject.getBnProjectLightValue();
			BnProjectValue pl = mProject.getBnProjectValue();
			StringWriter sw = new StringWriter();
			Marshaller.marshal(pl, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	 * Get node information
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getNodeDetails(String node) throws HeroException {

		try {
			BnNodeLocal nd = mProject.getBnNode(node);
			BnNodeLightValue nl = nd.getBnNodeLightValue();

			StringWriter sw = new StringWriter();			
			Marshaller.marshal(nl, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	 * Get node role information
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getNodeRole(String node) throws HeroException {

		try {
			BnNodeLocal nd = mProject.getBnNode(node);
			BnRoleValue rl = ((BnRoleLocal) nd.getBnRole()).getBnRoleValue();

			StringWriter sw = new StringWriter();
			Marshaller.marshal(rl, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	 * Get node executor information
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getNodeExecutor(String node) throws HeroException {

		try {
			BnNodeLocal nd = mProject.getBnNode(node);
			BnUserLightValue ul = ((BnUserLocal) nd.getExecutor()).getBnUserLightValue();

			StringWriter sw = new StringWriter();			
			Marshaller.marshal(ul, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	 * Get all nodes of this project
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getNodes() throws HeroException {

		ProjectNodes nb = new ProjectNodes();
		Collection nodes = new ArrayList();
		ArrayList ns = new ArrayList();
		StringWriter sw = new StringWriter();

		nodes = mProject.getBnNodes();
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			BnNodeLocal n = (BnNodeLocal) i.next();
			StrutsNodeValue snv = new StrutsNodeValue();
			snv.setProjectName(mProject.getName());
			snv.setName(n.getName());
			snv.setCreator(n.getCreator().getName());
			snv.setRole(n.getBnRole().getName());
			snv.setState(Constants.Nd.nodeStateName[n.getState()]);
			snv.setType(Constants.Nd.nodeTypeName[n.getType()]);
			if (n.getDescription() != null)
				snv.setDescription(n.getDescription());
			if (n.getExecutor() != null)
				snv.setExecutor(n.getExecutor().getName());
			snv.setAnticipable(n.getAnticipable());
			if (n.getDeadline() != null)
				snv.setDeadline(n.getDeadline().toString());
						
			ns.add(snv);			
		}
		nb.setNodes(ns.toArray());

		try {
			Marshaller.marshal(nb, sw);
		} catch (MarshalException e) {
			e.printStackTrace();
		} catch (ValidationException e) {
			e.printStackTrace();
		}
		return (sw.toString());
	}

	/**
	 * Get all users of this project
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getUsers() throws HeroException {

		ProjectUsers ub = new ProjectUsers();
		Collection us = new ArrayList();

		try {
			Collection users = mProject.getBnUsers();
			Iterator i = users.iterator();
			while (i.hasNext()) {
				BnUserLocal u = (BnUserLocal) i.next();
				BnUserLightValue ul = u.getBnUserLightValue();
				us.add(ul);
			}
			ub.setUser(us);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(ub, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	 * Get all roles of this project
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getRoles() throws HeroException {

		ProjectRoles rb = new ProjectRoles();
		Collection rs = new ArrayList();

		try {
			Collection roles = mProject.getBnRoles();
			Iterator i = roles.iterator();
			while (i.hasNext()) {
				BnRoleLocal r = (BnRoleLocal) i.next();
				BnRoleValue rl = r.getBnRoleValue();
				rs.add(rl);
			}
			rb.setRole(rs);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(rb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	 * Get user roles for this project
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getUserRoles(String userName) throws HeroException {

		log.debug("getUserRoles:");
		BnUserLocalHome uHome;
		BnUserLocal user;
		Collection result = new ArrayList();
		ProjectRoles rb = new ProjectRoles();
		try {
			uHome = (BnUserLocalHome) BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			user = uHome.findByName(userName);
		} catch (FinderException f) {
			throw new HeroException("FinderException");
		}
		Collection userRoles = user.getBnRoles();
		Collection projectRoles = mProject.getBnRoles();
		for (Iterator i = userRoles.iterator(); i.hasNext();) {
			BnRoleLocal ur = (BnRoleLocal) i.next();
			for (Iterator j = projectRoles.iterator(); j.hasNext();) {
				BnRoleLocal up = (BnRoleLocal) j.next();
				if (ur.getId() == up.getId())
					result.add(ur.getBnRoleValue());
			}
		}
		try {
			rb.setRole(result);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(rb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	 * Get the properties of this project
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getProperties() throws HeroException {

		ProjectProperties pb = new ProjectProperties();
		Collection ps = new ArrayList();

		try {
			Collection properties = mProject.getBnProperties();
			Iterator i = properties.iterator();
			while (i.hasNext()) {
				BnProjectPropertyLocal pp = (BnProjectPropertyLocal) i.next();
				BnProjectPropertyValue ppv = pp.getBnProjectPropertyValue();
				ps.add(ppv);
			}
			pb.setProperty(ps);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(pb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	 * Get the hooks of this project
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getHooks() throws HeroException {

		ProjectHooks hb = new ProjectHooks();
		Collection hs = new ArrayList();

		try {
			Collection hooks = mProject.getBnHooks();
			Iterator i = hooks.iterator();
			while (i.hasNext()) {
				BnProjectHookLocal ph = (BnProjectHookLocal) i.next();
				BnProjectHookValue phv = ph.getBnProjectHookValue();
				hs.add(phv);
			}
			hb.setHook(hs);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(hb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	 * Get the node properties of this project
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getNodeProperties(String nodeName) throws HeroException {

		NodeProperties nb = new NodeProperties();
		Collection ns = new ArrayList();

		try {
			BnNodeLocal nl = mProject.getBnNode(nodeName);
			Collection properties = nl.getBnProperties();
			Iterator i = properties.iterator();
			while (i.hasNext()) {
				BnNodePropertyLocal np = (BnNodePropertyLocal) i.next();
				BnNodePropertyValue npv = np.getBnNodePropertyValue();
				ns.add(npv);
			}
			nb.setProjectName(mProject.getName());
			nb.setProperty(ns);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(nb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	 * Get the node hooks of this project
	 *
	 * @ejb:interface-method view-type="both"
         * @ejb:transaction type="Supports"
	 *
	 **/

	public String getNodeHooks(String nodeName) throws HeroException {
		NodeHooks nb = new NodeHooks();
		Collection hs = new ArrayList();

		try {
			BnNodeLocal nl = mProject.getBnNode(nodeName);
			Collection hooks = nl.getBnHooks();
			Iterator i = hooks.iterator();
			while (i.hasNext()) {
				BnNodeHookLocal nh = (BnNodeHookLocal) i.next();
				BnNodeHookValue nhv = nh.getBnNodeHookValue();
				hs.add(nhv);
			}
			nb.setProjectName(mProject.getName());
			nb.setHook(hs);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(nb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	* get all in edges of the node
	* @return String
	*
	* @param name the name ot the node
	*
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Supports"
	*
	**/
	public String getNodeInEdges(String name) throws HeroException {

		NodeInEdges eb = new NodeInEdges();
		Collection in = new ArrayList();

		try {
			BnNodeLocal nd = mProject.getBnNode(name);

			Collection edgesIn = nd.getInBnEdges();
			Iterator i = edgesIn.iterator();
			while (i.hasNext())
				in.add(((BnEdgeLocal) i.next()).getBnEdgeValue());

			eb.setProjectName(mProject.getName());
			eb.setInEdge(in);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(eb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	* get all out edges of the node
	* @return Collection 
	*
	* @param name the name ot the node
	*
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Supports"
	*
	**/
	public String getNodeOutEdges(String name) throws HeroException {

		NodeOutEdges eb = new NodeOutEdges();
		Collection out = new ArrayList();

		try {
			BnNodeLocal nd = mProject.getBnNode(name);

			Collection edgesOut = nd.getOutBnEdges();
			Iterator i = edgesOut.iterator();
			while (i.hasNext())
				out.add(((BnEdgeLocal) i.next()).getBnEdgeValue());

			eb.setProjectName(mProject.getName());
			eb.setOutEdge(out);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(eb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}

	/**
	* import all the nodes of a project
	*
	* @param projectname the name of the project
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	*
	**/
	public void importProject(String projectName) throws HeroException {
		log.debug("importProject: project=" + projectName);
		importProject(projectName, "");
	}

	/**
	* import all the nodes of a project
	*
	* @param projectname the name of the project
	* @param prefix the prefix of names for the project
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	*
	**/
	public void importProject(String projectName, String prefix) throws HeroException {
		log.debug("importProject: project=" + projectName + " prefix=" + prefix);
		try {
			BnProjectLocalHome opHome = BnProjectUtil.getLocalHome();
			BnProjectLocal oldProject = opHome.findByName(projectName);

			//BnProject Hooks
			Collection hooks = oldProject.getBnHooks();
			for (Iterator i = hooks.iterator(); i.hasNext();) {
				BnProjectHookLocal hk = (BnProjectHookLocal) i.next();
				this.addHook(hk.getName(), hk.getEvent(), hk.getType());
			}

			BnRoleValue[] roles = oldProject.getBnProjectValue().getBnRoles();
			for (int r = 0; r < roles.length; r++) {
				String roleName = roles[r].getName();
				if (!containsRole(roleName)) {
					this.addRole(roleName, roles[r].getDescription());
				}
			}

			BnNodeValue[] nodes = oldProject.getBnProjectValue().getBnNodes();

			for (int n = 0; n < nodes.length; n++) {
				String newNodeName = prefix + nodes[n].getName();
				this.addNode(newNodeName, Constants.Nd.AND_JOIN_NODE);
				this.setNodeRole(newNodeName, nodes[n].getBnRole().getName());
				this.setNodeDescription(newNodeName, nodes[n].getDescription());

				//BnNode Hooks
				BnNodeLocal nd = oldProject.getBnNode(nodes[n].getName());
				Collection nodeHooks = nd.getBnHooks();
				for (Iterator j = hooks.iterator(); j.hasNext();) {
					BnNodeHookLocal nh = (BnNodeHookLocal) j.next();
					this.addNodeHook(newNodeName, nh.getName(), nh.getEvent(), nh.getType());
				}

			}

			BnEdgeValue[] edges = oldProject.getBnProjectValue().getBnEdges();

			for (int e = 0; e < edges.length; e++) {
				this.addEdge(prefix + (edges[e].getInBnNode()).getName(), prefix + (edges[e].getOutBnNode()).getName());
			}
		} catch (FinderException e) {
			throw new HeroException("Cannot find project " + e.getMessage());
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}

	}

	/**
	* init the BnProject Session Bean for SOAP interface only
	*
	* @throws HeroException 
	* @param projectName project name 
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	**/
	public void initProject(String projectName) throws HeroException {
		Context lContext;
		BnUserLocalHome uhome = null;
		BnRoleLocalHome rhome = null;
		log.debug("initProject: project=" + projectName);
		try {
			pHome = BnProjectUtil.getLocalHome();
			uhome = BnUserUtil.getLocalHome();
			rhome = BnRoleUtil.getLocalHome();
			mProject = pHome.findByName(projectName);
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			// The project has to be created
			try {
				String creator = mContext.getCallerPrincipal().getName();
				mProject = pHome.create(projectName, creator);
				BnUserLocal user = uhome.findByName(creator);
				mProject.getBnUsers().add(user);
				log.debug("Add user " + user.getBnUserLightValue().toString());
				// role admin is added
				BnRoleValue rv = new BnRoleValue();
				rv.setName(Constants.ADMIN);
				rv.setDescription("the admin role");
				BnRoleLocal role = rhome.create(rv);
				mProject.getBnRoles().add(role);
				this.setUserRole(creator, Constants.ADMIN);
				// role InitialRole is added
				rv = new BnRoleValue();
				rv.setName(Constants.INITIALROLE);
				rv.setDescription("the initial role for nodes");
				role = rhome.create(rv);
				mProject.getBnRoles().add(role);
				jms.sendProjectEvent(ADDUSERPROJECT, projectName, creator);
			} catch (FinderException fie) {
				throw new HeroException(fie.getMessage());
			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			} catch (Exception he) {
				throw new HeroException(he.getMessage());
			}
		}
	}

	/**
	* init the BnProject Session Bean for SOAP interface only (clone project)
	*
	* @throws HeroException 
	* @param oldProject name
	* @param newProject name 
	* @ejb:interface-method view-type="both"
        * @ejb:transaction type="Required"
	**/
	public void initProject(String oldProject, String newProject) throws CreateException {
		log.debug("initProject: oldProject=" + oldProject + " newProject=" + newProject);
		Context lContext;
		BnUserLocalHome uhome = null;
		BnRoleLocalHome rhome = null;

		try {
			pHome = BnProjectUtil.getLocalHome();
			uhome = BnUserUtil.getLocalHome();
			rhome = BnRoleUtil.getLocalHome();
			mProject = pHome.findByName(newProject);
		} catch (javax.naming.NamingException ne) {
			throw new CreateException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			// The project has to be created
			try {
				String creator = mContext.getCallerPrincipal().getName();
				mProject = pHome.create(newProject, creator);
				BnUserLocal user = uhome.findByName(creator);
				mProject.getBnUsers().add(user);
				log.debug("Add user " + user.getBnUserLightValue().toString());
				// role admin is added
				BnRoleValue rv = new BnRoleValue();
				rv.setName(Constants.ADMIN);
				rv.setDescription("the admin role");
				BnRoleLocal role = rhome.create(rv);
				mProject.getBnRoles().add(role);
				this.setUserRole(creator, Constants.ADMIN);
				// role InitialRole is added
				rv = new BnRoleValue();
				rv.setName(Constants.INITIALROLE);
				rv.setDescription("the initial role for nodes");
				role = rhome.create(rv);
				mProject.getBnRoles().add(role);
				importProject(oldProject);

				jms.sendProjectEvent(ADDUSERPROJECT, newProject, creator);

			} catch (FinderException fie) {
				throw new CreateException(fie.getMessage());
			} catch (InvalidValueException ie) {
				throw new CreateException(ie.getMessage());
			} catch (HeroException he) {
				throw new CreateException(he.getMessage());
			}
		}

	}

	/**
	* Create the BnProject Session Bean for SOAP interface only
	*
	* @throws CreateException 
	*
	* @ejb:create-method view-type="both"
	**/
	public void ejbCreate() throws CreateException {
	}

	/**
	* Get the BnProjectValue (historique)
	*
	* @ejb:interface-method view-type="both"
	*
	**/
	public BnProjectValue getDetails() throws HeroException {
		log.debug("getDetails:");
		return mProject.getBnProjectValue();
	}

	public void ejbPostCreate(String oldProject, String newProject) throws CreateException, RemoteException {
	}

	public void setSessionContext(final javax.ejb.SessionContext context) {
		mContext = context;
		jms = new JMSEvents();
	}

	public void ejbRemove() {
	}

	public void ejbActivate() {
	}

	public void ejbPassivate() {
	}

	// Utility methods

	private boolean checkDynamics(String in, String out) throws HeroException {
		BnNodeLocal nodein = mProject.getBnNode(in);
		BnNodeLocal nodeout = mProject.getBnNode(out);
		if ((nodeout.getState() == Constants.Nd.INITIAL) || (nodeout.getState() == Constants.Nd.READY) || (nodeout.getState() == Constants.Nd.ANTICIPABLE))
			return true;
		return false;
	}

	private boolean checkStructuralConstraints() throws HeroException {
		BnProjectValue pv = mProject.getBnProjectValue();
		return true;
	}

	private boolean checkDuplicateEdge(String in, String out) throws HeroException {
		Collection edges = mProject.getBnEdges();
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnEdgeLocal edge = (BnEdgeLocal) i.next();
			if ((edge.getInBnNode().getName()).equals(in) && (edge.getOutBnNode().getName()).equals(out))
				return false;
		}
		return true;
	}

	// Actually deletes an edge and restore the state of the outgoing node
	// Not a business method
	public void edgeDeletion(BnEdgeLocal ed) throws HeroException {
		BnNodeLocal nd = ed.getOutBnNode();
		if (nd.isExecuting()) {
			throw new DeleteEdgeException("Cannot delete edge. Out node executing");
		}
		if (nd.isTerminated()) {
			throw new DeleteEdgeException("Cannot delete edge. Out node executing");
		}
		try {
			ed.remove();
			int newOperation = nd.getActivation().state(nd);
			int newState = nd.getTransition().computeState(nd, newOperation);
			if (newState == Constants.Nd.BAD_TRANSITION) {
				throw new HeroException("AddEdge generated a bad BnNode transition");
			}
			this.setNodeState(nd, newState);
		} catch (RemoveException e) {
			throw new HeroException("deleteNode failed " + e.getMessage());
		}
	}

}

